# coding: utf-8

"""
    PyLucid models
    ~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os
import errno
import codecs
import mimetypes

from django.conf import settings
from django.db.models import signals
from django.db import models, IntegrityError
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse, NoReverseMatch
from django.contrib.sites.models import Site

# http://code.google.com/p/django-tools/
from django_tools.template import render

from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel, AutoSiteM2M
from pylucid_project.apps.pylucid.shortcuts import failsafe_message
from pylucid_project.apps.pylucid.system import headfile

from pylucid_project.pylucid_plugins import update_journal

# other PyLucid models
from colorscheme import ColorScheme, Color
from design import Design


TAG_INPUT_HELP_URL = \
"http://google.com/search?q=cache:django-tagging.googlecode.com/files/tagging-0.2-overview.html#tag-input"



class EditableHtmlHeadFileManager(models.Manager):
    def get_HeadfileLink(self, filename):
        """
        returns a pylucid.system.headfile.Headfile instance
        """
        db_instance = self.get(filename=filename)
        return headfile.HeadfileLink(filename=db_instance.filename)#, content=db_instance.content)


class EditableHtmlHeadFile(AutoSiteM2M, UpdateInfoBaseModel):
    """
    Storage for editable static text files, e.g.: stylesheet / javascript.

    TODO:
    * Check if Design sites in Headfile sites:
        This can't be done in save() method or in signals.post_save, because the related
        objects would be saved later -> It must be done in ManyRelatedManager.
        We should check if django ticked #5390 is done: 
            http://code.djangoproject.com/ticket/5390 # Add signals to ManyRelatedManager

    inherited attributes from AutoSiteM2M:
        sites   -> ManyToManyField to Site
        on_site -> sites.managers.CurrentSiteManager instance

    inherited attributes from UpdateInfoBaseModel:
        createtime     -> datetime of creation
        lastupdatetime -> datetime of the last change
        createby       -> ForeignKey to user who creaded this entry
        lastupdateby   -> ForeignKey to user who has edited this entry
    """
    objects = EditableHtmlHeadFileManager()

    filepath = models.CharField(max_length=255)
    mimetype = models.CharField(max_length=64)
    html_attributes = models.CharField(max_length=256, null=False, blank=True,
        # TODO: Use this!
        help_text='Additional html tag attributes (CSS example: media="screen")'
    )
    render = models.BooleanField(default=False,
        help_text="Are there CSS ColorScheme entries in the content?"
    )
    description = models.TextField(null=True, blank=True)
    content = models.TextField()

    def get_filename(self):
        """ returns only the filename """
        return os.path.split(self.filepath)[1]

    def get_color_filepath(self, colorscheme=None):
        """ Colorscheme + filepath """
        if colorscheme:
            assert isinstance(colorscheme, ColorScheme)
            return os.path.join("ColorScheme_%s" % colorscheme.pk, self.filepath)
        else:
            # The Design used no colorscheme
            return self.filepath

    def get_path(self, colorscheme):
        """ Path for filesystem cache path and link url. """
        return os.path.join(
            settings.PYLUCID.PYLUCID_MEDIA_DIR, settings.PYLUCID.CACHE_DIR,
            self.get_color_filepath(colorscheme)
        )

    def get_cachepath(self, colorscheme):
        """
        filesystem path with filename.
        TODO: Install section sould create the directories!
        """
        return os.path.join(settings.MEDIA_ROOT, self.get_path(colorscheme))

    def get_rendered(self, colorscheme):
        color_dict = Color.objects.get_color_dict(colorscheme)
        return render.render_string_template(self.content, color_dict)

    def save_cache_file(self, colorscheme):
        """
        Try to cache the head file into filesystem (Only worked, if python process has write rights)
        Try to create the out path, if it's not exist.
        """
        cachepath = self.get_cachepath(colorscheme)

        def _save_cache_file(auto_create_dir=True):
            rendered_content = self.get_rendered(colorscheme)
            try:
                f = codecs.open(cachepath, "w", "utf8")
                f.write(rendered_content)
                f.close()
            except IOError, err:
                if auto_create_dir and err.errno == errno.ENOENT: # No 2: No such file or directory
                    # Try to create the out dir and save the cache file
                    path = os.path.dirname(cachepath)
                    if not os.path.isdir(path):
                        # Try to create cache path and save file
                        os.makedirs(path)
                        failsafe_message("Cache path %s created" % path)
                        _save_cache_file(auto_create_dir=False)
                        return
                raise

        try:
            _save_cache_file()
        except (IOError, OSError), err:
            failsafe_message("Can't cache EditableHtmlHeadFile into %r: %s" % (cachepath, err))
        else:
            if settings.DEBUG:
                failsafe_message("EditableHtmlHeadFile cached successful into: %r" % cachepath)

    def save_all_color_cachefiles(self):
        """
        this headfile was changed: resave all cache files in every existing colors
        TODO: Update Queyset lookup
        """
        designs = Design.objects.all()
        for design in designs:
            headfiles = design.headfiles.all()
            for headfile in headfiles:
                if headfile == self:
                    colorscheme = design.colorscheme
                    self.save_cache_file(colorscheme)

    def get_absolute_url(self, colorscheme):
        cachepath = self.get_cachepath(colorscheme)
        if os.path.isfile(cachepath):
            # The file exist in media path -> Let the webserver send this file ;)
            return os.path.join(settings.MEDIA_URL, self.get_path(colorscheme))
        else:
            # not cached into filesystem -> use pylucid.views.send_head_file for it
            url = reverse('PyLucid-send_head_file', kwargs={"filepath":self.filepath})
            if colorscheme:
                return url + "?ColorScheme=%s" % colorscheme.pk
            else:
                # Design used no colorscheme
                return url

    def get_headfilelink(self, colorscheme):
        """ Get the link url to this head file. """
        url = self.get_absolute_url(colorscheme)
        return headfile.HeadfileLink(url)

    def auto_mimetype(self):
        """ returns the mimetype for the current filename """
        fileext = os.path.splitext(self.filepath)[1].lower()
        if fileext == ".css":
            return u"text/css"
        elif fileext == ".js":
            return u"text/javascript"
        else:
            mimetypes.guess_type(self.filepath)[0] or u"application/octet-stream"

    def save(self, *args, **kwargs):
        """
        TODO: update if model-validation branch merged into django
        """
        try:
            # "validate" the filepath with the url re. 
            reverse('PyLucid-send_head_file', kwargs={"filepath": self.filepath})
        except NoReverseMatch, err:
            raise ValidationError("filepath %r contains invalid characters!" % self.filepath)

        if self.id == None: # new item should be created.
            # manually check a unique togeher, because django can't do this with a M2M field.
            # Obsolete if unique_together work with ManyToMany: http://code.djangoproject.com/ticket/702
            exist = EditableHtmlHeadFile.on_site.filter(filepath=self.filepath).count()
            if exist != 0:
                # We can use attributes from this model instance, because it needs to have a primary key
                # value before a many-to-many relationship can be used.
                site = Site.objects.get_current()
                raise IntegrityError(
                    "EditableHtmlHeadFile with same filepath exist on site %r" % site
                )

        if not self.mimetype:
            # autodetect mimetype
            self.mimetype = self.auto_mimetype()

        # Try to cache the head file into filesystem (Only worked, if python process has write rights)
        self.save_all_color_cachefiles()

        return super(EditableHtmlHeadFile, self).save(*args, **kwargs)

    def __unicode__(self):
        sites = self.sites.values_list('name', flat=True)
        return u"'%s' (on sites: %r)" % (self.filepath, sites)

    class Meta:
        app_label = 'pylucid'
        #unique_together = ("filepath", "site")
        # unique_together doesn't work with ManyToMany: http://code.djangoproject.com/ticket/702
        ordering = ("filepath",)


#______________________________________________________________________________

def cache_headfiles(sender, **kwargs):
    """
    One colorscheme was changes: resave all cache headfiles with new color values.
    """
    colorscheme = kwargs["instance"]

    designs = Design.objects.all().filter(colorscheme=colorscheme)
    for design in designs:
        headfiles = design.headfiles.all()
        for headfile in headfiles:
            headfile.save_cache_file(colorscheme)

signals.post_save.connect(cache_headfiles, sender=ColorScheme)



#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from django.core.urlresolvers import reverse
from django.test.client import Client

from pylucid_project.tests.test_tools import basetest



class AdminTest(basetest.BaseLanguageTestCase):
    ADMIN_INDEX_URL = reverse("admin:index")

    def setUp(self):
        self.client = Client() # start a new session

    def assertAdminLoginPage(self, response):
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                # django
                'form action="/admin/" method="post"',
                # from pylucid:
                '$("form").submit',
                '$("form").find(":submit")'
            ),
            must_not_contain=("Traceback",)
        )

    def test_admin_login(self):
        """ request the normal django admin login page """
        response = self.client.get(self.ADMIN_INDEX_URL)
        self.assertAdminLoginPage(response)

    def test_admin_index(self):
        self.login(usertype="superuser")
        response = self.client.get("/admin/")
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                '<title>PyLucid - Site administration</title>',
                '<a href="/pylucid_admin/install/pylucid/">install PyLucid</a>',
                '<a href="/pylucid_admin/install/plugins/">install plugins</a>',
            ),
            must_not_contain=("Traceback",
                # django
                'form action="/admin/" method="post"',
                # from pylucid:
                '$("form").submit',
                '$("form").find(":submit")'
            )
        )

    def test_install_plugins(self):
        self.login(usertype="superuser")
        response = self.client.get("/pylucid_admin/install/plugins/")
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                '<title>PyLucid - PyLucid - Plugin install</title>',
                '*** Install Plugins:',

                "PyLucidAdminPage &#39;new content page&#39;"
                " (&#39;/pylucid_admin/plugins/page_admin/new_content_page/&#39;)&gt; created.",
            ),
            must_not_contain=("Traceback",)
        )

    def test_summary_page(self):
        self.login(usertype="superuser")
        response = self.client.get(self.ADMIN_INDEX_URL)
        self.assertResponse(response,
            must_contain=("PyLucid", "PageTree", "PageContent", "PageMeta"),
            must_not_contain=("Log in", "Traceback",)#"error")
        )

    def test_anonymous_add(self):
        """ Try to create a PageTree entry as a anonymous user. """
        url = reverse("admin:pylucid_pagetree_add")
        response = self.client.get(url)
        self.assertAdminLoginPage(response)

    def test_lang_german(self):
        """ Check if we get the admin page in the right language. """
        self.login(usertype="superuser")
        response = self.client.get(self.ADMIN_INDEX_URL, HTTP_ACCEPT_LANGUAGE="de")
        self.assertResponse(response,
            must_contain=(
                "PyLucid", "PageTree", "PageContent", "PageMeta",
                "Website-Verwaltung", "Kürzliche Aktionen"
            ),
            must_not_contain=("Log in", "Traceback",)
        )

if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
    management.call_command('test', __file__, verbosity=1)



#-----------------------------------------------------------------------------
# PyLucid bootstrap script START

import os, subprocess

def after_install(options, home_dir):
    etc = os.path.join(home_dir, 'etc')
    if not os.path.exists(etc):
        os.makedirs(etc)

    easy_install = os.path.join(home_dir, 'bin', 'easy_install')

    subprocess.call([easy_install, '--always-copy', 'pip'])

# PyLucid bootstrap script END
#-----------------------------------------------------------------------------


# coding: utf-8

"""
    PyLucid OpenStreetMap plugin
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~
    $LastChangedDate:$
    $Rev:$
    $Author: JensDiemer $

    :copyleft: 2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

__version__ = "$Rev:$"

from django.conf import settings
from django.utils.translation import ugettext as _

from pylucid_project.apps.pylucid.decorators import render_to

from OpenStreetMap.models import MapEntry


@render_to("OpenStreetMap/OpenStreetMap.html")
def lucidTag(request, name=None):
    """
    Display a map inline.
    First create a map entry in django admin panel.
    Then use the name for the lucidTag parameter.
    
    example:
        {% lucidTag OpenStreetMap name="MyMapName" %}
    """
    if name is None:
        if settings.DEBUG or request.user.is_staff:
            request.page_msg.error(_("lucidTag OpenStreetMap error: You must add the 'name' parameter!"))
        return "[OpenStreetMap Error]"
    
    try:
        map_entry = MapEntry.objects.get(name=name)
    except MapEntry.DoesNotExist:
        if settings.DEBUG or request.user.is_staff:
            request.page_msg.error(
                _("lucidTag OpenStreetMap error:"
                  " There exist no map entry with the name: %r") % name
            )
            existing_names = MapEntry.objects.values_list('name', flat=True)
            request.page_msg(_("Existing maps are: %r") % existing_names)
        return "[OpenStreetMap Error]"
    
    context = {
        "map":map_entry,
        "lang_code": request.PYLUCID.current_language.code, 
    }
    return context


#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittest
    ~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: 2009-05-12 16:47:00 +0200 (Di, 12. Mai 2009) $
    $Rev: 1975 $
    $Author: JensDiemer $

    :copyleft: 2009-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE.txt for more details.
"""

import os, unittest

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from django.conf import settings



class FakePageMsg(object):
    def __call__(self, *txt):
        print txt

class TestApplyMarkup(unittest.TestCase):
    """
    reverse test
    ~~~~~~~~~~~~
    Test the url re patterns via django.core.urlresolvers.reverse
    """
    def test_html(self):
        from pylucid_project.apps.pylucid.markup.converter import apply_markup
        from pylucid_project.apps.pylucid.models import PageContent
        content = apply_markup(raw_content="<h1>Hello World</h1>", markup_no=PageContent.MARKUP_HTML,
            page_msg=FakePageMsg(), escape_django_tags=False
        )
        self.failUnlessEqual(content, "<h1>Hello World</h1>")

    def test_creole(self):
        from pylucid_project.apps.pylucid.markup.converter import apply_markup
        from pylucid_project.apps.pylucid.models import PageContent
        raw_content = (
            u"test\n"
            "<<code=.html>>\n"
            "<h1>Hello World</h1>"
            "{% lucidTag foo %}\n"
            "<</code>>"
        )
        content = apply_markup(raw_content, markup_no=PageContent.MARKUP_CREOLE,
            page_msg=FakePageMsg(), escape_django_tags=False
        )
        self.failUnlessEqual(content,
            u'<p>test</p>\n<fieldset class="pygments_code">\n'
            '<legend class="pygments_code">HTML</legend>'
            '<table class="pygmentstable"><tr>'
            '<td class="linenos"><div class="linenodiv"><pre>1</pre></div></td>'
            '<td class="code"><div class="pygments">'
            '<pre>'
            '<span class="nt">&lt;h1&gt;</span>Hello World<span class="nt">&lt;/h1&gt;</span>'
            '&#x7B;% lucidTag foo %&#x7D;\n'
            '</pre>'
            '</div>\n</td></tr></table></fieldset>\n\n'

        )


if __name__ == "__main__":
    # Run this unittest directly
    from django.core import management
    management.call_command('test', __file__, verbosity=1)



# coding:utf-8

import os
from datetime import datetime
from tempfile import gettempdir

from datetime import datetime, timedelta
from django.utils.tzinfo import FixedOffset
from django.conf import settings
from django.db import connection
from django.core.cache import cache
from django.utils.translation import ugettext as _

from pylucid_project.utils.SimpleStringIO import SimpleStringIO
from pylucid_project.utils.timezone import utc_offset

from pylucid_project.apps.pylucid.models import Language, PageTree, PageMeta, LogEntry
from pylucid_project.apps.pylucid.decorators import check_permissions, render_to

from pylucid_project.apps.pylucid_admin.admin_menu import AdminMenu


MYSQL_ENCODING_VARS = (
    "character_set_server", "character_set_connection", "character_set_results", "collation_connection",
)


def install(request):
    """ insert PyLucid admin views into PageTree """
    output = []

    admin_menu = AdminMenu(request, output)
    menu_section_entry = admin_menu.get_or_create_section("system")

    admin_menu.add_menu_entry(
        parent=menu_section_entry,
        name="base check", title="A basic system setup check.",
        url_name="System-base_check"
    )
    admin_menu.add_menu_entry(
        parent=menu_section_entry,
        name="timezone info", title="INformation about timezone settings.",
        url_name="System-timezone"
    )

    return "\n".join(output)

#-----------------------------------------------------------------------------

def _cache_backend_test(request, out):
    out.write(_("\tsettings.CACHE_BACKEND is '%s'") % settings.CACHE_BACKEND)

    if settings.CACHE_BACKEND.startswith("dummy") or settings.CACHE_BACKEND.startswith("locmem"):
        out.write(_("\tPlease setup CACHE_BACKEND in you local_settings.py!"))
        out.write(_("\tmore info: http://docs.djangoproject.com/en/dev/topics/cache/#setting-up-the-cache"))
        tempdir = gettempdir()
        out.write(_("\te.g.: CACHE_BACKEND='file://%s'") % os.path.join(tempdir, "PyLucid_cache"))
        return

    cache_key = "cache test"
    content = "A cache test content..."
    cache_timeout = 50
    cache.set(cache_key, content, cache_timeout)
    cached_content = cache.get(cache_key)
    if cached_content == None:
        out.write(_("\t* Get None back. Cache didn't work!"))
        return
    elif cached_content == content:
        out.write(_("\t* Cache works fine ;)"))
    else:
        # Should never appears
        out.write(_("\t* Error! Cache content not the same!"))

    cache.delete(cache_key)
    cached_content = cache.get(cache_key)
    if cached_content != None:
        out.write(_("\t* Error: entry not deleted!"))

#-----------------------------------------------------------------------------

def _database_encoding_test(request, out):
    """
    Simple database encoding test:
        insert a test string into the database and check if 
        it is the same if we get the same entry back
    Use the PyLucid log table
    """
    out.write("Info: Not all database engines passed all tests!")

    def _test(range_txt, chr_range):
        out.write("\t%s test:" % range_txt)
        TEST_STRING = u"".join([unichr(i) for i in chr_range])
        try:
            log_entry1 = LogEntry.objects.log_action(
                "pylucid_plugin.system", "Database encoding test", request,
                message="%s test" % range_txt, long_message=TEST_STRING
            )
        except Warning, err:
            out.write("\t\tError: get a warning: %s" % err)
            return

        log_entry_id = log_entry1.id
        log_entry2 = LogEntry.objects.get(id=log_entry_id)
        if TEST_STRING == log_entry2.long_message:
            log_entry2.message += " - passed"
            out.write("\t\ttest passed")
        else:
            out.write("\t\ttest *NOT* passed")
            log_entry2.message += " - failed"
        log_entry2.save()

    _test("ASCII (32-126)", xrange(32, 126))
    _test("latin-1 (128-254)", xrange(128, 254))
    _test("ASCII control characters (0-31)", xrange(0, 31))
    _test("unicode plane 1-3 (0-12286 in 16 steps)", xrange(0, 12286, 16))
    _test("all unicode planes (0-65534 in 256 steps)", xrange(0, 65534, 256))



#-----------------------------------------------------------------------------

@check_permissions(superuser_only=True)
@render_to("system/base_check.html")
def base_check(request):
    out = SimpleStringIO()

    if settings.DEBUG:
        out.write(
            "*** Error: settings.DEBUG in on!"
            " (Should be off in productive environments!)"
        )
        out.write("\tSee: http://docs.djangoproject.com/en/dev/ref/settings/#debug")
    else:
        out.write("settings.DEBUG, ok")
    out.write("- "*40)


    if settings.PYLUCID.I18N_DEBUG:
        out.write(
            "*** Error: pylucid app_settings.I18N_DEBUG is on!"
            " (Should be off in productive environments!)"
        )
    else:
        out.write("pylucid app_settings.I18N_DEBUG, ok.")
    out.write("- "*40)


    if settings.SECRET_KEY == "":
        out.write(
            "*** Error: settings.SECRET_KEY not set!"
            " (You should add it into local-settings.py!)"
        )
        out.write("\tSee: http://docs.djangoproject.com/en/dev/ref/settings/#secret-key")
    else:
        out.write("settings.SECRET_KEY, ok.")
    out.write("- "*40)

    if settings.DATABASE_ENGINE == "mysql":
        try:
            import MySQLdb
            out.write("MySQLdb.__version__  : %s" % repr(MySQLdb.__version__))
            out.write("MySQLdb.version_info : %s" % repr(MySQLdb.version_info))
        except Exception, err:
            out.write("MySQLdb info error: %s" % err)

        cursor = connection.cursor()
        out.write("\nSome MySQL encoding related variables:")

        for var_name in MYSQL_ENCODING_VARS:
            cursor.execute("SHOW VARIABLES LIKE %s;", (var_name,))
            raw_result = cursor.fetchall()
            try:
                result = raw_result[0][1]
            except IndexError, err:
                out.write("%30s: Error: %s (raw result: %r)" % (var_name, err, raw_result))
            else:
                out.write("%30s: %s" % (var_name, result))
        out.write("- "*40)


    out.write("\nDatabase unicode test:")
    _database_encoding_test(request, out)
    out.write("- "*40)

    out.write("\nTest cache backend:")
    _cache_backend_test(request, out)
    out.write("- "*40)

    try:
        lang_entry = Language.objects.get(code=settings.LANGUAGE_CODE)
    except Language.DoesNotExist, err:
        out.write("*** Error: LANGUAGE_CODE %r doesn't exist!" % settings.LANGUAGE_CODE)
        languages = Language.objects.values_list("code", flat=True)
        out.write("\tExisting languages are: %r" % languages)
        out.write("\tset/change LANGUAGE_CODE in local-settings.py or create language %r." % settings.LANGUAGE_CODE)
    else:
        out.write("settings.LANGUAGE_CODE, ok.")
    out.write("- "*40)

    out.write("Check if all PageTree has at lease a PageMeta instance in the default system language:")
    default_lang_entry = Language.objects.get_or_create_default(request)
    exist_all = True
    # TODO: Can we but this into a big QuerySet?
    for pagetree in PageTree.on_site.all():
        exist = PageMeta.objects.filter(pagetree=pagetree, language=default_lang_entry)
        if not exist:
            exist_all = False
            out.write(
                "*** Error: PageTree entry %r has no PageMeta in default language (%r)!!!" % (
                    pagetree, default_lang_entry
                )
            )
    if exist_all:
        out.write("ok.")
    out.write("- "*40)

    context = {
        "title": "Basic system setup check",
        "results": out.getlines(),
    }
    return context




@check_permissions(superuser_only=True)
@render_to("system/timezone.html")
def timezone(request):
    """
    Display some informations about timezone
    """
    # get a timestamp from Django ORM datetime with 'auto_now_add'
    temp_log_entry = LogEntry.objects.log_action("pylucid_plugin.system", "timezone test", request)
    auto_now_add = temp_log_entry.createtime
    temp_log_entry.delete()

    context = {
        "datetime_now": datetime.now(),
        "datetime_utcnow": datetime.utcnow(),
        "auto_now_add": auto_now_add,
        "settings_TIME_ZONE": settings.TIME_ZONE,
        "environ_TZ": os.environ.get("TZ", "----"),
        "utc_offset": utc_offset(),
    }
    return context




# coding: utf-8

"""
    PyLucid plugin tools
    ~~~~~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: JensDiemer $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.p

"""

__version__ = "$Rev:$"

import re
import sys
import warnings

from django import http
from django.conf import settings
from django.template import loader
from django.http import HttpResponse
from django.core import urlresolvers
from django.utils.encoding import smart_str
from django.utils.importlib import import_module
from django.conf.urls.defaults import patterns, url

from pylucid_project.apps.pylucid.models import PluginPage, PageTree
from pylucid_project.apps.pylucid.system import pylucid_objects


from dbpreferences.forms import DBPreferencesBaseForm

class PyLucidDBPreferencesBaseForm(DBPreferencesBaseForm):
    def get_preferences(self, request, lucidtag_kwargs):
        """
        Update the preferences dict with the given kwargs dict.
        Send a staff user feedback if a kwargs key is invalid.
        """
        preferences = super(PyLucidDBPreferencesBaseForm, self).get_preferences()
        if request.user.is_staff:
            for key in lucidtag_kwargs.keys():
                if key not in preferences:
                    request.page_msg.info(
                        "Keyword argument %r is invalid for lucidTag %r !" % (key, self.Meta.app_label)
                    )
        preferences.update(lucidtag_kwargs)
        return preferences





class PluginGetResolver(object):
    def __init__(self, resolver):
        self.resolver = resolver
    def __call__(self, *args, **kwargs):
        return self.resolver

def _raise_resolve_error(plugin_url_resolver, rest_url):
    tried = [i[0][0][0] for i in plugin_url_resolver.reverse_dict.values()]
#    for key, value in plugin_url_resolver.reverse_dict.values():
#        print key, value

#    tried = [prefix + pattern.regex.pattern.lstrip("^") for pattern in plugin_urlpatterns]
    raise urlresolvers.Resolver404, {'tried': tried, 'path': rest_url + "XXX"}


def call_plugin(request, url_lang_code, prefix_url, rest_url):
    """ Call a plugin and return the response. """
    lang_entry = request.PYLUCID.current_language
    pluginpage = request.PYLUCID.pluginpage
    pagemeta = request.PYLUCID.pagemeta

    # build the url prefix
    # Don't use pagemeta.language.code here, use the real url_lang_code, because of case insensitivity
    url_prefix = "^%s/%s" % (url_lang_code, prefix_url)

    # Get pylucid_project.system.pylucid_plugins instance
    plugin_instance = pluginpage.get_plugin()

    plugin_url_resolver = plugin_instance.get_plugin_url_resolver(
        url_prefix, urls_filename=pluginpage.urls_filename,
    )
    #for key, value in plugin_url_resolver.reverse_dict.items(): print key, value

    # get the plugin view from the complete url
    resolve_url = request.path_info
    result = plugin_url_resolver.resolve(resolve_url)
    if result == None:
        _raise_resolve_error(plugin_url_resolver, resolve_url)

    view_func, view_args, view_kwargs = result

    if "pylucid.views" in view_func.__module__:
        # The url is wrong, it's from PyLucid and we can get a loop!
        # FIXME: How can we better check, if the view is from the plugin and not from PyLucid???
        _raise_resolve_error(plugin_url_resolver, resolve_url)

    merged_url_resolver = plugin_instance.get_merged_url_resolver(
        url_prefix, urls_filename=pluginpage.urls_filename,
    )

    # Patch urlresolvers.get_resolver() function, so only our own resolver with urlpatterns2
    # is active in the plugin. So the plugin can build urls with normal django function and
    # this urls would be prefixed with the current PageTree url.
    old_get_resolver = urlresolvers.get_resolver
    urlresolvers.get_resolver = PluginGetResolver(merged_url_resolver)

    # Call the view
    response = view_func(request, *view_args, **view_kwargs)

    # restore the patched function
    urlresolvers.get_resolver = old_get_resolver

    return response


#______________________________________________________________________________
# ContextMiddleware functions

TAG_RE = re.compile("<!-- ContextMiddleware (.*?) -->", re.UNICODE)
from django.utils.importlib import import_module
from django.utils.functional import memoize

_middleware_class_cache = {}

def _get_middleware_class(plugin_name):
    plugin_name = plugin_name.encode('ascii') # check non-ASCII strings

    mod_name = "pylucid_plugins.%s.context_middleware" % plugin_name
    module = import_module(mod_name)
    middleware_class = getattr(module, "ContextMiddleware")
    return middleware_class
_get_middleware_class = memoize(_get_middleware_class, _middleware_class_cache, 1)


def context_middleware_request(request):
    """
    get from the template all context middleware plugins and call the request method.
    """
    context = request.PYLUCID.context
    context["context_middlewares"] = {}

    page_template = request.PYLUCID.page_template # page template content
    plugin_names = TAG_RE.findall(page_template)
#    request.page_msg("Found ContextMiddlewares in content via RE: %r" % plugin_names)

    for plugin_name in plugin_names:
        # Get the middleware class from the plugin
        try:
            middleware_class = _get_middleware_class(plugin_name)
        except ImportError, err:
            request.page_msg.error("Can't import context middleware '%s': %s" % (plugin_name, err))
            continue

        # make a instance 
        instance = middleware_class(request, context)
        # Add it to the context
        context["context_middlewares"][plugin_name] = instance
#        request.page_msg("Init ContextMiddleware %r" % plugin_name)

def context_middleware_response(request, response):
    """
    replace the context middleware tags in the response, with the plugin render output
    """
    context = request.PYLUCID.context
    context_middlewares = context["context_middlewares"]
    def replace(match):
        plugin_name = match.group(1)
        try:
            middleware_class_instance = context_middlewares[plugin_name]
        except KeyError, err:
            return "[Error: context middleware %r doesn't exist! Existing middlewares are: %r]" % (
                plugin_name, context_middlewares.keys()
            )

        # Add info for pylucid_project.apps.pylucid.context_processors.pylucid
        request.plugin_name = plugin_name
        request.method_name = "ContextMiddleware"

        middleware_response = middleware_class_instance.render()

        request.plugin_name = None
        request.method_name = None

        if middleware_response == None:
            return ""
        elif isinstance(middleware_response, unicode):
            return smart_str(middleware_response, encoding=settings.DEFAULT_CHARSET)
        elif isinstance(middleware_response, str):
            return middleware_response
        elif isinstance(middleware_response, http.HttpResponse):
            return middleware_response.content
        else:
            raise RuntimeError(
                "plugin context middleware render() must return"
                " http.HttpResponse instance or a basestring or None!"
            )

    # FIXME: A HttpResponse allways convert unicode into string. So we need to do that here:
    # Or we say, context render should not return a HttpResponse?
#    from django.utils.encoding import smart_str
#    complete_page = smart_str(complete_page)

    source_content = response.content

    new_content = TAG_RE.sub(replace, source_content)
    response.content = new_content
    return response




# coding: utf-8

"""
    PyLucid.admin
    ~~~~~~~~~~~~~~

    Register all PyLucid model in django admin interface.

    TODO:
        * if http://code.djangoproject.com/ticket/3400 is implement:
            Add site to list_filter for e.g. PageMeta, PageContent etc.      
    
    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author$

    :copyleft: 2008 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.utils.translation import ugettext_lazy as _
from django.contrib import admin

from reversion.admin import VersionAdmin

from pylucid import models
from pylucid_project.apps.pylucid.base_admin import BaseAdmin

from pylucid_project.apps.pylucid_admin.admin_site import pylucid_admin_site


class PageTreeAdmin(BaseAdmin, VersionAdmin):
    #prepopulated_fields = {"slug": ("title",)}
    list_display = (
        "id", "parent", "slug", "showlinks", "site", "view_on_site_link", "lastupdatetime", "lastupdateby"
    )
    list_display_links = ("id", "slug")
    list_filter = (
        "site", "page_type", "permitViewGroup", "showlinks", "createby", "lastupdateby", "design",
    )
    date_hierarchy = 'lastupdatetime'
    search_fields = ("slug",)

pylucid_admin_site.register(models.PageTree, PageTreeAdmin)


class BanEntryAdmin(admin.ModelAdmin):
    list_display = list_display_links = ("ip_address", "createtime",)
    search_fields = ("ip_address",)
pylucid_admin_site.register(models.BanEntry, BanEntryAdmin)


class LanguageAdmin(VersionAdmin):
    list_display = ("code", "description", "site_info", "permitViewGroup")
    list_display_links = ("code", "description")
    list_filter = ("permitViewGroup",)
pylucid_admin_site.register(models.Language, LanguageAdmin)


class LogEntryAdmin(BaseAdmin):
    list_display = ("createtime", "createby", "view_on_site_link", "app_label", "action", "message")
    list_filter = (
        "site", "app_label", "action", "createby"
    )
    search_fields = ("app_label", "action", "message", "long_message", "data")
pylucid_admin_site.register(models.LogEntry, LogEntryAdmin)


#class OnSitePageMeta(models.PageMeta):
#    def get_site(self):
#        return self.page.site
#    site = property(get_site)
#    class Meta:
#        proxy = True



class PageMetaAdmin(BaseAdmin, VersionAdmin):
    list_display = ("id", "get_title", "get_site", "view_on_site_link", "lastupdatetime", "lastupdateby",)
    list_display_links = ("id", "get_title")
    list_filter = ("language", "createby", "lastupdateby", "tags")#"keywords"
    date_hierarchy = 'lastupdatetime'
    search_fields = ("description", "keywords")

pylucid_admin_site.register(models.PageMeta, PageMetaAdmin)


class PageContentInline(admin.StackedInline):
    model = models.PageContent

class PageContentAdmin(BaseAdmin, VersionAdmin):
    list_display = ("id", "get_title", "get_site", "view_on_site_link", "lastupdatetime", "lastupdateby",)
    list_display_links = ("id", "get_title")
    list_filter = ("markup", "createby", "lastupdateby",)
    date_hierarchy = 'lastupdatetime'
    search_fields = ("content",) # it would be great if we can add "get_title"

pylucid_admin_site.register(models.PageContent, PageContentAdmin)


class PluginPageAdmin(BaseAdmin, VersionAdmin):
    list_display = (
        "id", "get_plugin_name", "app_label",
        "get_site", "view_on_site_link", "lastupdatetime", "lastupdateby",
    )
    list_display_links = ("get_plugin_name", "app_label")
    list_filter = ("createby", "lastupdateby",)
    date_hierarchy = 'lastupdatetime'
    search_fields = ("app_label",)

pylucid_admin_site.register(models.PluginPage, PluginPageAdmin)

#-----------------------------------------------------------------------------

#class ColorAdmin(VersionAdmin):
#    list_display = ("id", "name","value")
#pylucid_admin_site.register(models.Color, ColorAdmin)

class ColorInline(admin.TabularInline):
    model = models.Color
    extra = 0

class ColorSchemeAdmin(VersionAdmin):
    list_display = ("id", "name", "preview", "site_info", "lastupdatetime", "lastupdateby")
    list_display_links = ("name",)
    search_fields = ("name",)
    list_filter = ("sites",)
    inlines = [ColorInline, ]

    def preview(self, obj):
        colors = models.Color.objects.all().filter(colorscheme=obj)
        result = ""
        for color in colors:
            result += '<span style="background-color:#%s;" title="%s">&nbsp;&nbsp;&nbsp;</span>' % (
                color.value, color.name
            )
        return result
    preview.short_description = 'color preview'
    preview.allow_tags = True

pylucid_admin_site.register(models.ColorScheme, ColorSchemeAdmin)


class DesignAdmin(VersionAdmin):
    list_display = ("id", "name", "template", "colorscheme", "site_info", "lastupdatetime", "lastupdateby")
    list_display_links = ("name",)
    list_filter = ("sites", "template", "colorscheme", "createby", "lastupdateby")
    search_fields = ("name", "template", "colorscheme")

pylucid_admin_site.register(models.Design, DesignAdmin)


class EditableHtmlHeadFileAdmin(VersionAdmin):
    list_display = ("id", "filepath", "site_info", "render", "description", "lastupdatetime", "lastupdateby")
    list_display_links = ("filepath", "description")
    list_filter = ("sites", "render")

pylucid_admin_site.register(models.EditableHtmlHeadFile, EditableHtmlHeadFileAdmin)

#-----------------------------------------------------------------------------

class UserProfileAdmin(VersionAdmin):
    list_display = ("id", "user", "site_info", "lastupdatetime", "lastupdateby")
    list_display_links = ("user",)
    list_filter = ("sites",)

pylucid_admin_site.register(models.UserProfile, UserProfileAdmin)



# coding: utf-8

"""
    PyLucid models
    ~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.db import models
from django.conf import settings

from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel, AutoSiteM2M
from pylucid_project.apps.pylucid.shortcuts import failsafe_message
from pylucid_project.apps.pylucid.fields import ColorValueField

from pylucid_project.pylucid_plugins import update_journal



TAG_INPUT_HELP_URL = \
"http://google.com/search?q=cache:django-tagging.googlecode.com/files/tagging-0.2-overview.html#tag-input"



class ColorScheme(AutoSiteM2M, UpdateInfoBaseModel):
    """
    inherited attributes from AutoSiteM2M:
        sites   -> ManyToManyField to Site
        on_site -> sites.managers.CurrentSiteManager instance
    """
    name = models.CharField(max_length=255, help_text="The name of this color scheme.")

    def update(self, colors):
        assert isinstance(colors, dict)
        new = []
        updated = []
        exists = []
        for name, value in colors.iteritems():
            color, created = Color.objects.get_or_create(
                colorscheme=self, name=name,
                defaults={"colorscheme":self, "name":name, "value":value}
            )
            if created:
                new.append(color)
            elif color.value != value:
                color.value = value
                updated.append(color)
                color.save()
            else:
                exists.append(color)
        return new, updated, exists

    def __unicode__(self):
        sites = self.sites.values_list('name', flat=True)
        return u"ColorScheme '%s' (on sites: %r)" % (self.name, sites)

    class Meta:
        app_label = 'pylucid'


class ColorManager(models.Manager):
    def get_color_dict(self, colorscheme):
        colors = self.all().filter(colorscheme=colorscheme)
        color_list = colors.values_list('name', 'value')
        return dict([(name, "#%s" % value) for name, value in color_list])

class Color(AutoSiteM2M, UpdateInfoBaseModel):
    """
    inherited attributes from AutoSiteM2M:
        sites   -> ManyToManyField to Site
        on_site -> sites.managers.CurrentSiteManager instance
    """
    objects = ColorManager()

    colorscheme = models.ForeignKey(ColorScheme)
    name = models.CharField(max_length=128,
        help_text="Name if this color (e.g. main_color, head_background)"
    )
    value = ColorValueField(help_text="CSS hex color value.")

    def save(self, *args, **kwargs):
        self.name = self.name.replace(" ", "_")
        new_name = self.name
        try:
            old_name = Color.objects.get(id=self.id).name
        except Color.DoesNotExist:
            # New color
            pass
        else:
            if new_name != old_name:
                from design import Design # import here, against import loops
                # Color name has been changed -> Rename template placeholder in every headfile, too.
                designs = Design.objects.all().filter(colorscheme=self.colorscheme)
                for design in designs:
                    headfiles = design.headfiles.all()
                    for headfile in headfiles:
                        if headfile.render != True: # File used no color placeholder
                            continue

                        old_content = headfile.content
                        # FIXME: Use flexibler regexp. for this:
                        new_content = old_content.replace("{{ %s }}" % old_name, "{{ %s }}" % new_name)
                        if old_content == new_content:
                            # content not changed?!?
                            failsafe_message(
                                "Color '{{ %s }}' not exist in headfile %r" % (old_name, headfile)
                            )
                            continue

                        if settings.DEBUG:
                            failsafe_message(
                                "change color name from '%s' to '%s' in %r" % (old_name, new_name, headfile)
                            )
                        headfile.content = new_content
                        headfile.save()

        return super(Color, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"Color '%s' #%s (%s)" % (self.name, self.value, self.colorscheme)

    class Meta:
        app_label = 'pylucid'
        unique_together = (("colorscheme", "name"),)
        ordering = ("colorscheme", "name")



# -*- coding: utf-8 -*-

"""
    PyLucid page update list plugin
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Generate a list of the latest page updates.

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: 2010-04-01 22:10:46 +0200 (Do, 01. Apr 2010) $
    $Rev: 2609 $
    $Author: JensDiemer $

    :copyleft: 2007-2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.p

"""

__version__ = "$Rev: 2609 $"


from django.conf import settings
from django.contrib.syndication.views import Feed
from django.utils.translation import ugettext_lazy as _
from django.utils.feedgenerator import Rss201rev2Feed, Atom1Feed
from django.core.urlresolvers import NoReverseMatch

from pylucid_project.apps.pylucid.decorators import render_to
from pylucid_project.utils.safe_obtain import safe_pref_get_integer
from pylucid_project.apps.pylucid.models import Language, PluginPage

from update_journal.models import UpdateJournal
from update_journal.preference_forms import UpdateJournalPrefForm


def _get_queryset(request, count):
    """ TODO: Move to UpdateJournal.objects ? """
    queryset = UpdateJournal.on_site.all()

    languages = request.PYLUCID.languages
    queryset = queryset.filter(language__in=languages)

    if not request.user.is_staff:
        queryset = queryset.filter(staff_only=False)

    return queryset[:count]


@render_to("update_journal/update_journal_table.html")
def lucidTag(request, count=10):
    try:
        count = int(count)
    except Exception, e:
        if request.user.is_staff:
            request.page_msg.error("page_update_list error: count must be a integer (%s)" % e)
        count = 10

    queryset = _get_queryset(request, count)

    try:
        select_feed_url = PluginPage.objects.reverse("update_journal", "UpdateJournal-select_feed")
    except NoReverseMatch, err:
        select_feed_url = None
        if not settings.DEBUG and request.user.is_staff:
            # PluginPage.objects.reverse creates a page_msg only in DEBUG mode.
            request.page_msg.error(err)

    context = {
        "update_list": queryset,
        "select_feed_url": select_feed_url
    }
    return context


class RssFeed(Feed):
    feed_type = Rss201rev2Feed
    filename = "feed.rss"

    title = _("Update Journal - RSS feed")
    link = "/"
    description_template = "update_journal/feed_description.html"

    def __init__(self, request):
        self.request = request

        # Get max number of feed entries from request.GET["count"]
        # Validate/Limit it with information from DBPreferences 
        self.count, error = safe_pref_get_integer(
            request, "count", UpdateJournalPrefForm,
            default_key="initial_feed_count", default_fallback=5,
            min_key="initial_feed_count", min_fallback=5,
            max_key="max_feed_count", max_fallback=30
        )

    def description(self):
        return _("Last %s updates and changes") % self.count

    def items(self):
        return _get_queryset(self.request, self.count)

    def item_title(self, item):
        return item.title

    def item_link(self, item):
        return item.object_url


class AtomFeed(RssFeed):
    """
    http://docs.djangoproject.com/en/dev/ref/contrib/syndication/#publishing-atom-and-rss-feeds-in-tandem
    """
    feed_type = Atom1Feed
    filename = "feed.atom"
    title = _("Update Journal - Atom feed")
    subtitle = RssFeed.description


# The last class is the fallback class, if filename doesn't match
FEEDS = [AtomFeed, RssFeed]


@render_to("update_journal/select_feed.html")
def select_feed(request):
    """
    Display a list with existing feed filenames.
    """
    context = {"feeds": FEEDS}
    return context


def feed(request, filename):
    """
    return RSS/Atom feed selected by filename.
    """
    #print filename
    for feed_class in FEEDS:
        if filename == feed_class.filename:
            break

    #print "feed class:", feed_class
    feed = feed_class(request)
    response = feed(request)
    return response



#!/usr/bin/env python
# coding: utf-8

"""
    Check some basic requirements
"""

import sys
import subprocess


class Tester(object):
    def __init__(self):
        self.ok = True

    def check(self, cmd, err_info):
        print " ".join(cmd)
        try:
            returncode = subprocess.call(cmd)
        except Exception, err:
            print "Error:", err
        else:
            if returncode == 0:
                print "OK"
                print
                return

        self.ok = False
        print "***", err_info
        print


if __name__ == "__main__":
    print "requirements pre test (%s)" % __file__
    print

    t = Tester()
    t.check(["python", "-V"], "Python not installed?!?!")
    t.check(["svn", "--version", "--quiet"],
        "Please install subversion! (e.g.: sudo aptitude install subversion)"
    )

    if t.ok == True:
        # All test ok
        sys.exit(0)
    else:
        # One test failed
        sys.exit(-1)




# coding: utf-8

"""
    TODO:
    * handel proxy's 'HTTP_X_FORWARDED_FOR' values.
        See notes here:
        http://docs.djangoproject.com/en/1.0/ref/middleware/#reverse-proxy-middleware
"""

import datetime

from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User
from django.contrib.admin.util import quote
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import smart_unicode
from django.utils.safestring import mark_safe
from django.conf import settings
from django.contrib.sites.models import Site
from django.contrib.sites.managers import CurrentSiteManager

# http://code.google.com/p/django-tools/
from django_tools.middlewares import ThreadLocal

# http://code.google.com/p/django-dbpreferences/
from dbpreferences.fields import DictField

from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel


META_KEYS = (
    "REMOTE_ADDR", "REMOTE_USER", "REQUEST_METHOD", "QUERY_STRING",
    "HTTP_REFERER", "HTTP_USER_AGENT", "HTTP_ACCEPT_ENCODING", "HTTP_ACCEPT_LANGUAGE"
)

class LogEntryManager(models.Manager):


    def get_same_remote_addr(self, request):
        """
        return a QuerySet with all entries from the current remote Address.
        """
        current_remote_addr = request.META["REMOTE_ADDR"]
        queryset = self.model.on_site.all()
        queryset = queryset.filter(remote_addr=current_remote_addr)
        return queryset

    def last_remote_addr_actions(self, request, seconds):
        """
        creates a queryset with all items from the same REMOTE_ADDR in the last seconds.
        """
        queryset = self.get_same_remote_addr(request)

        timedelta = datetime.timedelta(seconds=seconds)
        point_in_time = datetime.datetime.now() - timedelta
        queryset = queryset.filter(createtime__gte=point_in_time)

        return queryset

    def request_limit(self, request, min_pause, ban_limit, app_label, no_page_msg=False):
        """
        Monitor request min_pause and ban_limit.
        """
        # Count the last requests for this app_label
        queryset = self.last_remote_addr_actions(request, min_pause)
        queryset = queryset.filter(app_label=app_label)
        last_actions = queryset.count()

        if last_actions >= ban_limit:
            msg = _(
                "Add ban entry, because %(last_actions)s request for %(app_label)s"
                " in the last %(min_pause)ssec."
            ) % {
                "last_actions": last_actions,
                "app_label": app_label,
                "min_pause": min_pause,
            }
            self.log_action(app_label=app_label, action="ban ip", message=msg)

            from pylucid_project.apps.pylucid.models import BanEntry
            BanEntry.objects.add(request) # raise 404 after adding the client IP!

        if last_actions > 0:
            msg = _("Request too fast!")
            debug_msg = " (%s in the last %ssec. IP is blocked by %s overruns.)" % (
                last_actions, min_pause, ban_limit
            )
            if settings.DEBUG:
                msg += debug_msg
            if no_page_msg == False:
                request.page_msg.error(msg)
            LogEntry.objects.log_action(
                app_label=app_label, action="request aborted", message=debug_msg,
            )
            raise self.model.RequestTooFast(msg)

    def log_action(self, app_label, action, request=None, message=None, long_message=None, data=None):
        if request is None:
            request = ThreadLocal.get_current_request()

        kwargs = {
            "uri": request.build_absolute_uri(),
            "app_label": app_label,
            "action": action,
            "message": message,
            "long_message": long_message,
            "data": data,
        }

        if hasattr(request, "PYLUCID"):
            kwargs["used_language"] = request.PYLUCID.current_language

        for key in META_KEYS:
            value = request.META.get(key)
            if value and len(value) > 255:
                value = "%s..." % value[:252]
            kwargs[key.lower()] = value

        new_entry = self.model(**kwargs)
        new_entry.save()
        return new_entry


class LogEntry(UpdateInfoBaseModel):
    """
    PyLucid action log entries.

    inherited attributes from UpdateInfoBaseModel:
        createtime     -> datetime of creation
        lastupdatetime -> datetime of the last change
        createby       -> ForeignKey to user who creaded this entry
        lastupdateby   -> ForeignKey to user who has edited this entry
    """
    class RequestTooFast(Exception):
        pass

    objects = LogEntryManager()

    site = models.ForeignKey(Site, default=Site.objects.get_current)
    on_site = CurrentSiteManager()

    # Log information:
    app_label = models.CharField(_('App Label'), max_length=255, db_index=True,
        help_text="The App name witch has created this log entry."
    )
    action = models.CharField(_('Action'), max_length=128, db_index=True,
        help_text="Short action key. (e.g.: 'do search', 'login')"
    )
    message = models.CharField(_('Message'), max_length=255, blank=True, null=True,
        help_text="Short/one line log message. (e.g.: 'user FooBar login')"
    )
    long_message = models.TextField(_('long Message'), blank=True, null=True,
        help_text="Complete log message."
    )
    # From django-dbpreferences
    data = DictField(blank=True, null=True, help_text="serialized dictionary data")

    # Own meta data:
    uri = models.CharField(_('URL'), max_length=255,
        help_text="absolute URI form request.build_absolute_uri()"
    )
    used_language = models.ForeignKey("pylucid.Language", blank=True, null=True)

    # Data from request.META
    remote_addr = models.IPAddressField(_('Remote IP Address'), blank=True, null=True, db_index=True,
        help_text="The IP address of the client. From request.META['REMOTE_ADDR']"
    )
    remote_user = models.CharField(_('Remote User'), max_length=255, blank=True, null=True,
        help_text="The user authenticated by the web server, if any. From request.META['REMOTE_USER']"
    )
    request_method = models.CharField(_('Request Method'), max_length=8, blank=True, null=True,
        help_text="Request method, e.g.: 'GET', 'POST'. From request.META['REQUEST_METHOD']"
    )
    query_string = models.CharField(_('Query String'), max_length=255, blank=True, null=True,
        help_text="The query string, as a single **unparsed** string. From request.META['QUERY_STRING']"
    )
    http_referer = models.CharField(_('Referer'), max_length=255, blank=True, null=True,
        help_text="The referring page, if any. From request.META['HTTP_REFERER']"
    )
    http_user_agent = models.CharField(_('User Agent'), max_length=255, blank=True, null=True,
        help_text="The client's user-agent string. From request.META['HTTP_USER_AGENT']"
    )
    http_accept_encoding = models.CharField(_('Accept Encoding'), max_length=255, blank=True, null=True,
        help_text="from request.META['HTTP_ACCEPT_ENCODING']"
    )
    http_accept_language = models.CharField(_('Accept Language'), max_length=255, blank=True, null=True,
        help_text="from request.META['HTTP_ACCEPT_LANGUAGE']"
    )

    def get_absolute_url(self):
        return self.uri

    def __unicode__(self):
        return u"LogEntry %s %s %s" % (self.createby, self.createtime, self.action)

    class Meta:
        app_label = 'pylucid'
        verbose_name = _('log entry')
        verbose_name_plural = _('log entries')
        ordering = ('-createtime',)





# coding:utf-8

from django.db import models
from django.core.cache import cache
from django.core import urlresolvers
from django.utils.translation import ugettext as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User, Group, Permission

from django_tools.middlewares import ThreadLocal

from pylucid_project.apps.pylucid.tree_model import BaseTreeModel, TreeManager, TreeGenerator
from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel


class PyLucidAdminManager(TreeManager):
    def get_tree_for_user(self, user):
        filtered_items = self.get_for_user(user)
        tree = TreeGenerator(filtered_items)
        return tree

    def get_for_user(self, user):
        """
        returns only the menu items, for which the user has the rights.
        TODO: Filter menu sections, too. (If there is no sub items, remove the section)
        """
        all_items = self.all().order_by("position")
        filtered_items = []
        for item in all_items:
            superuser_only, access_permissions = item.get_permissions()
            if superuser_only == True and user.is_superuser == False:
                continue
            if not user.has_perms(access_permissions):
                continue

            filtered_items.append({
                "id": item.id,
                "parent": getattr(item.parent, "id", None),
                "absolute_url": item.get_absolute_url(),
                "name": item.name,
                "title": item.title,
            })

        return filtered_items

class PyLucidAdminPage(BaseTreeModel, UpdateInfoBaseModel):
    """
    PyLucid Admin page tree
    
    inherited attributes from TreeBaseModel:
        parent
        position

    inherited attributes from UpdateInfoBaseModel:
        createtime     -> datetime of creation
        lastupdatetime -> datetime of the last change
        createby       -> ForeignKey to user who creaded this entry
        lastupdateby   -> ForeignKey to user who has edited this entry
    """
    objects = PyLucidAdminManager()

    #TODO: check if url_name is unique. We can't set unique==True,
    #      because menu section has always url_name=None
    url_name = models.CharField(blank=True, null=True, max_length=256,
        help_text="Name of url, defined in plugin/admin_urls.py"
    )

    name = models.CharField(max_length=150, unique=True,
        help_text="Sort page name (for link text in e.g. menu)"
    )
    title = models.CharField(blank=True, null=False, max_length=256,
        help_text="A long page title (for e.g. page title or link title text)"
    )

    get_pagetree = models.BooleanField(default=False,
        verbose_name="get PageTree",
        help_text="Add current PageTree ID via GET Parameter to the url, if available"
    )
    get_pagemeta = models.BooleanField(default=False,
        verbose_name="get PageMeta",
        help_text="Add current PageMeta ID via GET Parameter to the url, if available"
    )
    get_page = models.BooleanField(default=False,
        verbose_name="get PageContent/PluginPage",
        help_text="Add current PageContent or current PluginPage ID via GET Parameter to the url, if available"
    )

    def get_permissions(self):
        """
        returns the access permissions for this menu entry.
        TODO: Should be cache this?
        """
        if not self.url_name: # a menu section
            return (False, ())

        # Get the view function for this url_name
        # FIXME: Can we get it faster and not with resolve the url?
        url = urlresolvers.reverse(self.url_name)
        view_func, func_args, func_kwargs = urlresolvers.resolve(url)

        # get the rights from pylucid_project.apps.pylucid.decorators.check_permissions
        access_permissions = view_func.permissions
        superuser_only = view_func.superuser_only

        return (superuser_only, access_permissions)

#    def save(self, *args, **kwargs):
#        """
#        After change, deletes panel_extra from cache.
#        cache filled in pylucid_plugins.admin_menu.views.panel_extras()
#        """
#        for user_id in User.objects.values_list('id', flat=True):
#            cache_key = "panel_extras_%s" % user_id
#            cache.delete(cache_key)
#
#        return super(PyLucidAdminPage, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"PyLucidAdminPage %r (%r)" % (self.name, self.get_absolute_url())

    def get_absolute_url(self):
        """
        absolute url (without domain/host part)
        TODO: Should be used a cache here?
        """
        if not self.url_name:
            return "" # menu section

        url = urlresolvers.reverse(viewname=self.url_name)
        request = ThreadLocal.get_current_request()
        get_data = {}
        if self.get_pagetree and hasattr(request.PYLUCID, "pagetree"):
            get_data["pagetree"] = request.PYLUCID.pagetree.pk
        if self.get_pagemeta and hasattr(request.PYLUCID, "pagemeta"):
            get_data["pagemeta"] = request.PYLUCID.pagemeta.pk
        if self.get_page:
            if hasattr(request.PYLUCID, "pagecontent"):
                get_data["pagecontent"] = request.PYLUCID.pagecontent.pk
            elif hasattr(request.PYLUCID, "pluginpage"):
                get_data["pluginpage"] = request.PYLUCID.pluginpage.pk

        if get_data:
            # FIXME: There must be a better was to to this.
            # TODO: escape it.
            url += "?" + "&".join(["%s=%s" % (key, value) for key, value in get_data.items()])

        return url

    class Meta:
        verbose_name = _('PyLucid admin page')
        verbose_name_plural = _('PyLucid admin pages')
        ordering = ("url_name",)



# coding: utf-8

"""
    PyLucid OpenStreetMap plugin
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~
    $LastChangedDate:$
    $Rev:$
    $Author: JensDiemer $

    :copyleft: 2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

from django.utils.translation import ugettext_lazy as _

from reversion.admin import VersionAdmin

from pylucid_project.apps.pylucid_admin.admin_site import pylucid_admin_site

from OpenStreetMap.models import MapEntry


class MapEntryAdmin(VersionAdmin):
    def lucidTag_example(self, obj):
        return '{%% lucidTag OpenStreetMap name="%s" %%}' % obj.name

    lucidTag_example.short_description = _("lucidTag example")
    lucidTag_example.allow_tags = False

    list_display = ("name", "lucidTag_example", "lon", "lat", "marker_text")
    list_display_links = ("name",)
    list_filter = ("createby", "lastupdateby",)
    date_hierarchy = 'lastupdatetime'
    search_fields = ("name", "marker_text")

pylucid_admin_site.register(MapEntry, MapEntryAdmin)



# coding: utf-8

"""
    PyLucid blog plugin
    ~~~~~~~~~~~~~~~~~~~

    A simple blog system.

    http://feedvalidator.org/
    
    TODO:
        * Detail view, use BlogEntry.get_absolute_url()
    

    Last commit info:
    ~~~~~~~~~
    $LastChangedDate: 2010-04-01 22:10:46 +0200 (Do, 01. Apr 2010) $
    $Rev: 2609 $
    $Author: JensDiemer $

    :copyleft: 2008-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

__version__ = "$Rev: 2609 $"


from django import http
from django.conf import settings
from django.contrib.syndication.views import Feed
from django.utils.translation import ugettext as _
from django.views.decorators.csrf import csrf_protect
from django.contrib.comments.views.comments import post_comment
from django.utils.feedgenerator import Rss201rev2Feed, Atom1Feed

from pylucid_project.apps.pylucid.system import i18n
from pylucid_project.apps.pylucid.decorators import render_to
from pylucid_project.utils.safe_obtain import safe_pref_get_integer

from blog.models import BlogEntry
from blog.preference_forms import BlogPrefForm

# from django-tagging
from tagging.models import Tag, TaggedItem



def _add_breadcrumb(request, *args, **kwargs):
    """ shortcut for add breadcrumb link """
    context = request.PYLUCID.context
    breadcrumb_context_middlewares = context["context_middlewares"]["breadcrumb"]
    breadcrumb_context_middlewares.add_link(*args, **kwargs)


def _get_queryset(request, tags=None, filter_language=False):
    # Get all blog entries, that the current user can see
    queryset = BlogEntry.objects.all_accessible(request, filter_language=filter_language)

    if tags is not None:
        # filter by tags 
        queryset = TaggedItem.objects.get_by_model(queryset, tags)

    return queryset


def _split_tags(raw_tags):
    "simple split tags from url"
    tags = raw_tags.strip("/").split("/")
    return tags


class RssFeed(Feed):
    feed_type = Rss201rev2Feed
    filename = "feed.rss"
    title = _("Blog - RSS feed")
    link = "/"
    description_template = "blog/feed_description.html"

    def __init__(self, request, tags=None):
        self.request = request

        if tags is not None:
            tags = _split_tags(tags)
        self.tags = tags

        # Get max number of feed entries from request.GET["count"]
        # Validate/Limit it with information from DBPreferences 
        self.count, error = safe_pref_get_integer(
            request, "count", BlogPrefForm,
            default_key="initial_feed_count", default_fallback=5,
            min_key="initial_feed_count", min_fallback=5,
            max_key="max_feed_count", max_fallback=30
        )

    def description(self):
        if self.tags is None:
            return _("Last %s blog articles") % self.count
        else:
            return _(
                 "Last %(count)s blog articles tagged with: %(tags)s"
            ) % {"count":self.count, "tags": ",".join(self.tags)}

    def items(self):
        queryset = _get_queryset(self.request, self.tags, filter_language=True)
        return queryset[:self.count]

    def item_title(self, item):
        return item.headline

    def item_author_name(self, item):
        return item.createby

    def item_link(self, item):
        return item.get_absolute_url()


class AtomFeed(RssFeed):
    """
    http://docs.djangoproject.com/en/dev/ref/contrib/syndication/#publishing-atom-and-rss-feeds-in-tandem
    """
    feed_type = Atom1Feed
    filename = "feed.atom"
    title = _("Blog - Atom feed")
    subtitle = RssFeed.description


# The last class is the fallback class, if filename doesn't match
FEEDS = [AtomFeed, RssFeed]



@render_to("blog/summary.html")
def summary(request):
    """
    Display summary list with all blog entries.
    """
    # Get all blog entries, that the current user can see
    queryset = _get_queryset(request, filter_language=True)

    # Limit the queryset with django Paginator
    paginator = BlogEntry.objects.paginate(request, queryset)

    tag_cloud = BlogEntry.objects.get_tag_cloud(request)

    current_lang = request.PYLUCID.current_language.description
    _add_breadcrumb(request, _("All blog articles in %s.") % current_lang)

    context = {
        "entries": paginator,
        "tag_cloud": tag_cloud,
        "CSS_PLUGIN_CLASS_NAME": settings.PYLUCID.CSS_PLUGIN_CLASS_NAME,
        "feeds": FEEDS,
    }
    return context


@render_to("blog/summary.html")
def tag_view(request, tags):
    """
    Display summary list with blog entries filtered by the given tags.
    """
    tags = _split_tags(tags)

    # Get all blog entries, that the current user can see
    queryset = _get_queryset(request, tags, filter_language=True)

    # Limit the queryset with django Paginator
    paginator = BlogEntry.objects.paginate(request, queryset)

    # Add link to the breadcrumbs ;)
    text = _("All items tagged with: %s") % ", ".join(tags)
    current_lang = request.PYLUCID.current_language.description
    _add_breadcrumb(request, text, text + _(" and are written in %s.") % current_lang)

    tag_cloud = BlogEntry.objects.get_tag_cloud(request)

    context = {
        "entries": paginator,
        "tag_cloud": tag_cloud,
        "add_tag_filter_link": True, # Add + tag filter link
        "CSS_PLUGIN_CLASS_NAME": settings.PYLUCID.CSS_PLUGIN_CLASS_NAME,
        "used_tags": tags,
        "tags": "/".join(tags),
        "feeds": FEEDS,
    }
    return context


@csrf_protect
@render_to("blog/detail_view.html")
def detail_view(request, id, title):
    """
    Display one blog entry with a comment form.
    """
    # Get all blog entries, that the current user can see
    queryset = _get_queryset(request, filter_language=False)

    try:
        entry = queryset.get(pk=id)
    except BlogEntry.DoesNotExist:
        # It's possible that the user comes from a external link.
        msg = "Blog entry doesn't exist."
        if settings.DEBUG or request.user.is_staff:
            msg += " (ID %r wrong.)" % id
        request.page_msg.error(msg)
        return summary(request)

    new_url = i18n.assert_language(request, entry.language)
    if new_url:
        # the current language is not the same as entry language -> redirect to right url
        # e.g. someone followed a external link to this article, but his preferred language
        # is a other language as this article. Activate the article language and "reload"
        return http.HttpResponsePermanentRedirect(new_url)

    # Add link to the breadcrumbs ;)
    _add_breadcrumb(request, entry.headline, _("Blog article '%s'") % entry.headline)

    if request.POST:
        # Use django.contrib.comments.views.comments.post_comment to handle a comment
        # post.
        return post_comment(request, next=entry.get_absolute_url())

    tag_cloud = BlogEntry.objects.get_tag_cloud(request)

    # Change permalink from the blog root page to this entry detail view
    permalink = entry.get_permalink(request)
    request.PYLUCID.context["page_permalink"] = permalink # for e.g. the HeadlineAnchor

    context = {
        "page_title": entry.headline, # Change the global title with blog headline
        "entry": entry,
        "tag_cloud": tag_cloud,
        "CSS_PLUGIN_CLASS_NAME": settings.PYLUCID.CSS_PLUGIN_CLASS_NAME,
        "page_permalink": permalink, # Change the permalink in the global page template
    }
    return context

#------------------------------------------------------------------------------


@render_to("blog/select_feed.html")
def select_feed(request):
    """ Display a list with existing feed filenames. """
    context = {"feeds": FEEDS}
    return context


def feed(request, filename, tags=None):
    """
    return RSS/Atom feed for all blog entries and filtered by tags. 
    Feed format is selected by filename.
    """
    for feed_class in FEEDS:
        if filename == feed_class.filename:
            break

    feed = feed_class(request, tags)
    response = feed(request)
    return response




# coding: utf-8

"""
    PyLucid models
    ~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.db import models
from django.conf import settings
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.utils.translation.trans_real import parse_accept_lang_header

# http://code.google.com/p/django-tools/
from django_tools.middlewares import ThreadLocal
from django_tools.fields import LanguageCodeField

from pylucid_project.apps.pylucid.shortcuts import failsafe_message
from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel, AutoSiteM2M

from pylucid_project.pylucid_plugins import update_journal


supported_languages = dict(settings.LANGUAGES)

TAG_INPUT_HELP_URL = \
"http://google.com/search?q=cache:django-tagging.googlecode.com/files/tagging-0.2-overview.html#tag-input"


ACCESSIBLE_LANG_CACHE = {}



class LanguageManager(models.Manager):
    def filter_accessible(self, queryset, user):
        """ filter all languages with can't accessible for the given user """

        if user.is_anonymous():
            # Anonymous user are in no user group
            return queryset.filter(permitViewGroup__isnull=True)

        if user.is_superuser:
            # Superuser can see everything ;)
            return queryset

        # filter pages for not superuser and not anonymous
        user_groups = user.groups.values_list('pk', flat=True)

        if not user_groups:
            # User is in no group
            return queryset.filter(permitViewGroup__isnull=True)

        # Filter out all view group
        return queryset.filter(
            models.Q(permitViewGroup__isnull=True) | models.Q(permitViewGroup__in=user_groups)
        )

    def all_accessible(self, user):
        """ returns a QuerySet of all languages that the given user can access. """
        queryset = self.model.on_site.all()
        queryset = self.filter_accessible(queryset, user)
        return queryset

    def get_cached_languages(self, user):
        """ returns a cached *list* of all languages that the given user can access. """
        cache_key = user
        if cache_key not in ACCESSIBLE_LANG_CACHE:
            languages = self.all_accessible(user)
#            if settings.PYLUCID.I18N_DEBUG:
#                failsafe_message("all accessible languages from db: %r" % languages)
            ACCESSIBLE_LANG_CACHE[cache_key] = list(languages)
#        elif settings.PYLUCID.I18N_DEBUG:
#                failsafe_message("all accessible languages from cache: %r" % ACCESSIBLE_LANG_CACHE[cache_key])

        return ACCESSIBLE_LANG_CACHE[cache_key][:]

    def get_choices(self):
        """ return a tuple list for e.g. forms.ChoiceField """
        return self.values_list('code', 'description')

    default_lang_entry = None
    def get_or_create_default(self, request):
        """
        return Languange instance with code from settings.LANGUAGE_CODE
        Create if not exist.
        """
        if self.default_lang_entry is None:
            language_code = settings.LANGUAGE_CODE

            self.default_lang_entry = self.get_from_code(request, language_code)
            if self.default_lang_entry is None:
                failsafe_message("Default language entry not in language list?")
                self.default_lang_entry, created = self.get_or_create(
                    code=language_code, defaults={'description': language_code}
                )
                if created:
                    failsafe_message("Default language entry %r created." % self.default_lang_entry)
        return self.default_lang_entry

    def _get_default_language(self):
        """
        return Languange instance with code from settings.LANGUAGE_CODE
        Should normaly not used! Use request.PYLUCID.default_language !
        e.g. needed in unittest
        """
        if self.default_lang_entry is None:
            language_code = settings.LANGUAGE_CODE
            self.default_lang_entry = self.model.on_site.get(code=language_code)
        return self.default_lang_entry

    def _get_language_codes(self, request):
        """
        Create a language code list.
         - use all client accepted languages
         - add current used language and system default language        
        """
        current_lang_code = getattr(request, "LANGUAGE_CODE", None)
        default_lang_code = settings.LANGUAGE_CODE

        accept_lang_codes = []
        unsupported_lang_codes = []
        fallback_lang_codes = []

        accept = request.META.get('HTTP_ACCEPT_LANGUAGE', '')
#        if settings.PYLUCID.I18N_DEBUG:
#            request.page_msg.info("HTTP_ACCEPT_LANGUAGE: %r" % accept)

        if not accept:
            if current_lang_code:
                accept_lang_codes = [current_lang_code, default_lang_code]
            else:
                accept_lang_codes = [default_lang_code]

            return accept_lang_codes, [], []

        accept_lang_headers = parse_accept_lang_header(accept)
#        print "accept_lang_headers:", accept_lang_headers
        for accept_lang, unused in accept_lang_headers:
            if "-" in accept_lang:
                # remember the first part for later adding as a fallback language
                fallback_lang = accept_lang.split("-", 1)[0]
                if fallback_lang in supported_languages and fallback_lang not in fallback_lang_codes:
                    fallback_lang_codes.append(fallback_lang)

            if accept_lang not in supported_languages:
                unsupported_lang_codes.append(accept_lang)
                continue

            if accept_lang not in accept_lang_codes:
                accept_lang_codes.append(accept_lang)

        if current_lang_code:
            # insert/move current language at the beginning
            if not accept_lang_codes:
                accept_lang_codes = [current_lang_code]
            elif accept_lang_codes[0] != current_lang_code:
                if current_lang_code in accept_lang_codes:
                    pos = accept_lang_codes.index(current_lang_code)
                    del(accept_lang_codes[pos])

                accept_lang_codes.insert(0, current_lang_code)

        # append fallback languages
        for fallback_lang in fallback_lang_codes:
            if fallback_lang not in accept_lang_codes:
                accept_lang_codes.append(fallback_lang)

        # append default language at the end
        if default_lang_code not in accept_lang_codes:
            accept_lang_codes.append(default_lang_code)

        return accept_lang_codes, unsupported_lang_codes, fallback_lang_codes

    def get_languages(self, request):
        """
        Create a list of all languages sorted by client accept language priority.
        
        added to request.PYLUCID.languages
        
        Cache key: The language entry can have a permitViewGroup,
            so we must use different caches for different users.
        """
        if hasattr(request, "PYLUCID") and hasattr(request.PYLUCID, "languages"):
            if settings.PYLUCID.I18N_DEBUG:
                request.page_msg.info(
                    "return request.PYLUCID.languages: %r" % request.PYLUCID.languages
                )
            return request.PYLUCID.languages

        user = request.user
        languages = self.get_cached_languages(user)

        if settings.PYLUCID.I18N_DEBUG:
            failsafe_message("all accessible languages: %r" % languages)

        accept_lang_codes, unsupported_lang_codes, fallback_lang_codes = self._get_language_codes(request)

        if settings.PYLUCID.I18N_DEBUG:
            request.page_msg.info("accept_lang_codes: %r" % accept_lang_codes)
            request.page_msg.info("unsupported_lang_codes: %r" % unsupported_lang_codes)
            request.page_msg.info("fallback_lang_codes: %r" % fallback_lang_codes)

        # XXX: Test QuerySet order
#            language_codes.sort()
#            language_codes.sort(reverse=True)

        # sort the language in the same order than language_codes list was.
        # XXX: It there a better way to do this?
        language_list = []
        for language_code in accept_lang_codes:
            for index, language in enumerate(languages):
                if language.code.lower() == language_code.lower():
                    if language not in language_list:
                        language_list.append(language)
                    del(languages[index])
                    break

        if languages:
            # The Client has not all existing languages in his HTTP_ACCEPT_LANGUAGE
            # Append the rest
            if settings.PYLUCID.I18N_DEBUG:
                request.page_msg.info(
                    "client not accepted languages to append: %s" % ", ".join([l.code for l in languages])
                )
            language_list += languages

        if settings.PYLUCID.I18N_DEBUG:
            request.page_msg.info("language_list: %s" % ", ".join([l.code for l in language_list]))

        return language_list

    def get_from_code(self, request, language_code):
        language_list = self.get_languages(request)
        for language in language_list:
            if language.code.lower() == language_code.lower():
                return language

    def get_current(self, request=None):
        """ return client Language instance, if not available, use get_default_lang_entry() """
        if request == None:
            request = ThreadLocal.get_current_request()

        if request == None:
            # no request available, e.g. loading fixtures
            return self._get_default_language()

        language_list = self.get_languages(request)
        return language_list[0]

#        if request:
#            if hasattr(request, "PYLUCID"):
#                return request.PYLUCID.current_language
#
#            if hasattr(request, "LANGUAGE_CODE"):
#                lang_code = request.LANGUAGE_CODE
#                if "-" in lang_code:
#                    lang_code = lang_code.split("-", 1)[0]
#                try:
#                    return self.get(code=lang_code)
#                except Language.DoesNotExist:
#                    if settings.PYLUCID.I18N_DEBUG:
#                        msg = (
#                            'Favored language "%s" does not exist -> use default lang from system preferences'
#                        ) % request.LANGUAGE_CODE
#                        failsafe_message(msg)
#
#        return self.get_or_create_default(request)




class Language(AutoSiteM2M, UpdateInfoBaseModel):
    """
    inherited attributes from AutoSiteM2M:
        sites   -> ManyToManyField to Site
        on_site -> sites.managers.CurrentSiteManager instance
        
    inherited attributes from UpdateInfoBaseModel:
        createtime     -> datetime of creation
        lastupdatetime -> datetime of the last change
        createby       -> ForeignKey to user who creaded this entry
        lastupdateby   -> ForeignKey to user who has edited this entry
    """
    objects = LanguageManager()

    code = LanguageCodeField(unique=True, max_length=10)
    description = models.CharField(max_length=150, blank=True,
        help_text="Description of the Language (filled automaticly)"
    )

    permitViewGroup = models.ForeignKey(Group, related_name="%(class)s_permitViewGroup",
        help_text="Limit viewable to a group for a complete language section?",
        null=True, blank=True,
    )

    def clean_fields(self, exclude):
        message_dict = {}

        if "code" not in exclude and self.code not in supported_languages:
            message_dict["code"] = (
                "Language is not in settings.LANGUAGES!",
                "Supported languages are: %s" % ",".join(sorted(supported_languages.keys()))
            )

        if message_dict:
            raise ValidationError(message_dict)

    def save(self, *args, **kwargs):
        global ACCESSIBLE_LANG_CACHE
        ACCESSIBLE_LANG_CACHE = {}

        if not self.description:
            self.description = supported_languages[self.code]

        super(Language, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"Language %s - %s" % (self.code, self.description)

    class Meta:
        app_label = 'pylucid'
        ordering = ("code",)



# coding: utf-8

"""
    Create/update the virtualenv bootstrap script BOOTSTRAP_SCRIPT from
    the BOOTSTRAP_SOURCE file.
"""

import sys

try:
    import virtualenv
except ImportError, err:
    print "Import error:", err
    print
    print "Please install virtualenv!"
    print "e.g.: easy_install virtualenv"
    print "More info:"
    print "http://pypi.python.org/pypi/virtualenv"
    print
    sys.exit()


BOOTSTRAP_SCRIPT = "pylucid-boot.py"
BOOTSTRAP_SOURCE = "source-pylucid-boot.py"


def create_bootstrap_script():
    info = "source bootstrap script: %r" % BOOTSTRAP_SOURCE
    print "read", info
    content = "# %s\n" % info
    f = file(BOOTSTRAP_SOURCE, "r")
    content += f.read()
    f.close()

    print "Create/Update %r" % BOOTSTRAP_SCRIPT

    output = virtualenv.create_bootstrap_script(content)

    # Add info lines
    output_lines = output.splitlines()
    output_lines.insert(2, "## Generate with %r" % __file__)
    output_lines.insert(2, "## using: %r" % virtualenv.__file__)
    output = "\n".join(output_lines)
    #print output

    f = file(BOOTSTRAP_SCRIPT, 'w')
    f.write(output)
    f.close()


if __name__ == "__main__":
    create_bootstrap_script()
    print " -- END -- "


# coding: utf-8

from django.conf import settings
from django.contrib import admin
from django.http import HttpResponseRedirect


class PyLucidAdminSite(admin.AdminSite):
    """
    Own PyLucid Admin Site.
    """
    def logout(self, request):
        if request.GET: # Don't redirect, if any GET parameter in url
            return super(PyLucidAdminSite, self).logout(request)

        # Use the PyLucid own logout view, so the user it back on his cms page
        url = "/?" + settings.PYLUCID.AUTH_LOGOUT_GET_VIEW
        return HttpResponseRedirect(url)


pylucid_admin_site = PyLucidAdminSite()


# Use all django contrib model admins in our own admin site ;)
pylucid_admin_site._registry = admin.site._registry



# coding: utf-8

"""
setup some "static" variables
"""

from django.conf import settings
from django.contrib.sites.models import Site
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

from dbtemplates.models import Template

from pylucid_project import VERSION_STRING
from pylucid_project.utils import slug


def _add_plugin_info(request, context):
    """ Add css anchor into context. Used information from lucidTagNode. """

    plugin_name = request.plugin_name
    method_name = request.method_name

    if not hasattr(request, "css_id_list"):
        request.css_id_list = []

    css_plugin_id = plugin_name + u"_" + method_name
    existing_slugs = request.css_id_list
    css_plugin_id = slug.makeUniqueSlug(css_plugin_id, existing_slugs)

    request.css_id_list.append(css_plugin_id)

    context["css_plugin_id"] = css_plugin_id
    context["css_plugin_class"] = plugin_name

    return context


def pylucid(request):
    """
    A django TEMPLATE_CONTEXT_PROCESSORS
    http://www.djangoproject.com/documentation/templates_python/#writing-your-own-context-processors
    """
    current_site = Site.objects.get_current()
    all_sites = Site.objects.all()

    context = {
        "powered_by": mark_safe('<a href="http://www.pylucid.org">PyLucid v%s</a>' % VERSION_STRING),
        # This value would be changed in index._render_cms_page(), if the
        # plugin manager or any plugin set request.anonymous_view = False
        "robots": "index,follow", # TODO: remove in v0.9, see: ticket:161

        "CSS_PLUGIN_CLASS_NAME": settings.PYLUCID.CSS_PLUGIN_CLASS_NAME,

        "current_site": current_site,
        "sites": all_sites,

        "PyLucid_media_url": settings.MEDIA_URL + settings.PYLUCID.PYLUCID_MEDIA_DIR + "/",
        "Django_media_prefix": settings.ADMIN_MEDIA_PREFIX,

        "debug": settings.DEBUG,
    }

    context["PYLUCID"] = request.PYLUCID

    pagetree = getattr(request.PYLUCID, "pagetree", None)
    if pagetree:
        template_name = pagetree.design.template
        context["template_name"] = template_name

        # Add the dbtemplates entry.
        # Used in pylucid_admin_menu.html for generating the "edit page template" link
        try:
            context["template"] = Template.on_site.get(name=template_name)
        except Template.DoesNotExist:
            context["template"] = None

    pagemeta = getattr(request.PYLUCID, "pagemeta", None)
    if pagemeta:
        context.update({
            "page_title": pagemeta.get_title(),
            "page_keywords": pagemeta.keywords,
            "page_description": pagemeta.description,
            "page_robots": pagemeta.robots,
            "page_language": pagemeta.language.code,
            "page_permalink": pagemeta.get_permalink(),
            "page_absolute_url": pagemeta.get_absolute_url(),
        })

    if getattr(request, "plugin_name", None) != None:
        # Add css anchor info
        context = _add_plugin_info(request, context)

    return context



# coding: utf-8

"""
    PyLucid models
    ~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.db import models
from django.conf import settings
from django.core import urlresolvers
from django.contrib.sites.models import Site

# http://code.google.com/p/django-tools/
from django_tools.utils import installed_apps_utils
from django_tools import model_utils

from pylucid_project.apps.pylucid.shortcuts import failsafe_message
from pylucid_project.system.pylucid_plugins import PYLUCID_PLUGINS
from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel, BaseModel, BaseModelManager


TAG_INPUT_HELP_URL = \
"http://google.com/search?q=cache:django-tagging.googlecode.com/files/tagging-0.2-overview.html#tag-input"



class RootAppChoiceField(models.CharField):
    def get_choices_default(self):
        PluginPage.objects.get_app_choices()

class PluginPageManager(BaseModelManager):
    """
    TODO: In next release witch has a update routine: we should switch:
        from plugin_instance.installed_apps_string to plugin_instance.pkg_string
    """
    _APP_CHOICES = None
    def get_app_choices(self):
        """
        Generate a choice list with all views witch can handle a empty root url.
        But PyLucid can only handle own plugins, see:
            http://trac.pylucid.net/ticket/333
        """
        if self._APP_CHOICES == None:
            root_apps = installed_apps_utils.get_filtered_apps(resolve_url="/")

            self._APP_CHOICES = [("", "---------")]
            for app in root_apps:
                plugin_name = app.split(".")[-1]
                if not plugin_name in PYLUCID_PLUGINS:
                    continue
                plugin_instance = PYLUCID_PLUGINS[plugin_name]

                self._APP_CHOICES.append(
                    (plugin_instance.installed_apps_string, plugin_instance.pkg_string)
                )

#            apps = [app for app in root_apps if not "pylucid_project.apps" in app]
#            self._APP_CHOICES = [("", "---------")] + [(app, app) for app in sorted(apps)]
        return self._APP_CHOICES

    def reverse(self, plugin_name, viewname, args=(), kwargs={}):
        """
        reverse a plugin url.
        Please note: this will always use the first PluginPage entry as url prefix!
        """
        # get the app label from
        plugin_instance = PYLUCID_PLUGINS[plugin_name]
        app_label = plugin_instance.installed_apps_string

        # Get the first PluginPage entry for this plugin
        queryset = PluginPage.objects.all()
        queryset = queryset.filter(pagetree__site=Site.objects.get_current())
        queryset = queryset.filter(app_label=app_label)
        try:
            plugin_page = queryset[0]
        except (IndexError, KeyError):
            msg = "Can't get a PluginPage for plugin %r, please create one." % plugin_name
            if settings.DEBUG:
                msg += " (app_label: %r)" % app_label
                failsafe_message(msg)
            raise urlresolvers.NoReverseMatch(msg)

        url_prefix = plugin_page.get_absolute_url()
        plugin_url_resolver = plugin_instance.get_plugin_url_resolver(url_prefix, plugin_page.urls_filename)
        return plugin_url_resolver.reverse(viewname, *args, **kwargs)


class PluginPage(BaseModel, UpdateInfoBaseModel):
    """
    A plugin page

    inherited attributes from UpdateInfoBaseModel:
        createtime     -> datetime of creation
        lastupdatetime -> datetime of the last change
        createby       -> ForeignKey to user who creaded this entry
        lastupdateby   -> ForeignKey to user who has edited this entry
    """
    objects = PluginPageManager()

    pagetree = models.OneToOneField("pylucid.PageTree")

    app_label = RootAppChoiceField(max_length=256,
        help_text=(
            "The plugin app label witch is in settings.INSTALLED_APPS"
            " (Only apps witch can handle a root url.)"
        )
    )
    urls_filename = models.CharField(max_length=256, default="urls.py",
        help_text="Filename of the urls.py"
    )

    def get_site(self):
        return self.pagetree.site

    def get_absolute_url(self):
        """ absolute url *with* language code (without domain/host part) """
        from pylucid_project.apps.pylucid.models import Language # import here against import loops

        pagetree_url = self.pagetree.get_absolute_url()
        lang_entry = Language.objects.get_current()
        url = "/" + lang_entry.code + pagetree_url
        return url

    def get_title(self):
        """ The page title is optional, if not exist, used the slug from the page tree """
        pagemeta = self.get_pagemeta()
        return pagemeta.get_title()

    def get_plugin_name(self):
        return self.app_label.split(".")[-1]

    def get_plugin(self):
        """ returns pylucid_project.system.pylucid_plugins instance """
        plugin_name = self.get_plugin_name()
        plugin_instance = PYLUCID_PLUGINS[plugin_name]
        return plugin_instance

    def save(self, *args, **kwargs):
        if not self.pagetree.page_type == self.pagetree.PLUGIN_TYPE:
            # FIXME: Better error with django model validation?
            raise AssertionError("Plugin can only exist on a plugin type tree entry!")
        return super(PluginPage, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"PluginPage '%s' (pagetree: %r)" % (self.app_label, self.pagetree)

    class Meta:
        app_label = 'pylucid'
        verbose_name_plural = verbose_name = "PluginPage"
        ordering = ("-lastupdatetime",)
#        ordering = ("pagetree", "language")

# Check Meta.unique_together manually
model_utils.auto_add_check_unique_together(PluginPage)



#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    TODO: Test colorscheme stuff
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from pylucid_project.tests.test_tools import basetest
from pylucid_project.tests.test_tools.scrapping import HTMLscrapper
from pylucid_project.apps.pylucid.models import EditableHtmlHeadFile, ColorScheme



class DesignTestCase(basetest.BaseUnittest):
    STYLES = ['initial_site_style/main.css', 'pygments.css'] # used on the index page

    def _pre_setup(self, *args, **kwargs):
        super(DesignTestCase, self)._pre_setup(*args, **kwargs)

        self.colorscheme = ColorScheme.objects.get(id=1) # used in index page

        self.headfiles = []
        for style_path in self.STYLES:
            headfile = EditableHtmlHeadFile.on_site.get(filepath=style_path)
            self.headfiles.append(headfile)

    def get_headlinks(self, response):
        data = HTMLscrapper().grab(response.content, tags=("link",), attrs=("href",))
        return [url for url in data["href"] if "headfile" in url]

    def assertHeadfiles(self, urls):
        for url in urls:
            response = self.client.get(url)
            self.assertStatusCode(response, 200)
            self.failUnlessEqual(response["content-type"], "text/css")

    def check_headfile(self, headfile, colorscheme):
        url = headfile.get_absolute_url(colorscheme)
        self.assertHeadfiles([url])


class DesignTest(DesignTestCase):

    def test_index_styles(self):
        """ grab style links from index page and test if they can be requested """
        response = self.client.get("/")
        urls = self.get_headlinks(response)
        self.failUnless(len(urls) == 2)
        self.assertHeadfiles(urls)

    def test_cache(self):
        """ Test cached and non cache requests """
        # Create the cache file, if not exist
        for headfile in self.headfiles:
            cachepath = headfile.get_cachepath(self.colorscheme)
            if os.path.isfile(cachepath):
                # remove cache file
                os.remove(cachepath)
                self.failIf(os.path.isfile(cachepath), "Can't delete %r ?!?!" % cachepath)

            # Check if headfile.get_absolute_url can be requested
            self.check_headfile(headfile, self.colorscheme)

            headfile.save() # The save method should create the cache file

            # Check if file exist
            self.failUnless(os.path.isfile(cachepath), "Cache file %r doesn't not exist?!?!" % cachepath)

            # Check if headfile.get_absolute_url can be requested
            self.check_headfile(headfile, self.colorscheme)



if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
    management.call_command('test', __file__, verbosity=1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    PyLucid CGI wrapper
    ~~~~~~~~~~~~~~~~~~~

    CGI wrapper for Django using the WSGI protocol.

    taken from http://code.djangoproject.com/ticket/2407
    original name: "cgi.py"

    Code copy/pasted from PEP-0333 and then tweaked to serve django.
    http://www.python.org/dev/peps/pep-0333/#the-server-gateway-side

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: 2009-07-24 17:05:37 +0200 (Fr, 24. Jul 2009) $
    $Rev: 2156 $
    $Author: JensDiemer $

    :copyright: 2007 by Jens Diemer
    :license: GNU GPL v3, see LICENSE.txt for more details.
"""

import os, sys
import django.core.handlers.wsgi

def runcgi():
    environ                      = dict(os.environ.items())
    environ['PATH_INFO']         = environ.get('PATH_INFO',"/")
    environ['wsgi.input']        = sys.stdin
    environ['wsgi.errors']       = sys.stderr
    environ['wsgi.version']      = (1,0)
    environ['wsgi.multithread']  = False
    environ['wsgi.multiprocess'] = True
    environ['wsgi.run_once']     = True

    application = django.core.handlers.wsgi.WSGIHandler()

    if environ.get('HTTPS','off') in ('on','1'):
        environ['wsgi.url_scheme'] = 'https'
    else:
        environ['wsgi.url_scheme'] = 'http'

    headers_set  = []
    headers_sent = []

    def send_content_type(response_headers):
        for no, header in enumerate(response_headers):
            if header[0].lower() == "content-type":
                sys.stdout.write('%s: %s\r\n' % header)
                del(response_headers[no])
                return response_headers

        sys.stdout.write('Content-Type: text/html\r\n')
        sys.stdout.write('Warning: Content Type not send!') # Bullshit?!?

    def write(data):
        if not headers_set:
            raise AssertionError("write() before start_response()")

        elif not headers_sent:
            # Before the first output, send the stored headers
            status, response_headers = headers_sent[:] = headers_set

            response_headers = send_content_type(response_headers) # Send Content-Type first

            if status == "500":
                sys.stdout.write("Content-type: text/html; charset=utf-8\r\n\r\nHARDCORE DEBUG:\n")
            sys.stdout.write('Status: %s\r\n' % status)
            for header in response_headers:
                sys.stdout.write('%s: %s\r\n' % header)
            sys.stdout.write('\r\n')

        sys.stdout.write(data)
        sys.stdout.flush()

    def start_response(status,response_headers,exc_info=None):
        if exc_info:
            try:
               if headers_sent:
                   # Re-raise original exception if headers sent
                   raise exc_info[0], exc_info[1], exc_info[2]
            finally:
               exc_info = None     # avoid dangling circular ref
        elif headers_set:
            raise AssertionError("Headers already set!")

        headers_set[:] = [status,response_headers]
        return write

    result = application(environ, start_response)
    try:
        for data in result:
            if data:    # don't send headers until body appears
                write(data)
        if not headers_sent:
            write('')   # send headers now if body was empty
    finally:
        if hasattr(result,'close'):
            result.close()



# coding: utf-8

"""
    PyLucid page admin
    ~~~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate:$
    $Rev:$
    $Author: JensDiemer $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.conf import settings
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.http import HttpResponse, HttpResponseRedirect

from pylucid_project.apps.pylucid.shortcuts import render_pylucid_response
from pylucid_project.apps.pylucid.markup.converter import apply_markup
from pylucid_project.apps.pylucid.decorators import check_permissions
from pylucid_project.apps.pylucid.models import PageTree, PageContent

from page_admin.forms import EditPageForm


def _get_pageobjects(request):
    """
    returns current PageContent instance.
    Note: In get views, the page content doesn't exist on PyLucid objects.
    """
    pagemeta = request.PYLUCID.pagemeta
    pagecontent = PageContent.objects.get(pagemeta=pagemeta)
    return pagemeta, pagecontent

def _edit_page(request, form_url):
    pagetree = request.PYLUCID.pagetree
    if pagetree.page_type != PageTree.PAGE_TYPE:
        request.page_msg("Current page is not a content page.")
        return

    pagemeta, pagecontent = _get_pageobjects(request)
    preview_html = ""

    if request.method == 'POST':
        edit_page_form = EditPageForm(request.POST)
        if edit_page_form.is_valid():
            if "preview" in request.POST:
                raw_content = edit_page_form.cleaned_data["content"]
                preview_html = apply_markup(
                    raw_content, pagecontent.markup, request.page_msg, escape_django_tags=True
                )
            else:
                new_content = edit_page_form.cleaned_data["content"]
                pagecontent.content = new_content
                pagecontent.save()
                request.page_msg.successful(_("Page content updated."))
                return HttpResponseRedirect(request.path)
    else:
        edit_page_form = EditPageForm(initial={"content":pagecontent.content})

    context = {
        "form_url": form_url,
        "abort_url": request.path,
        "preview_url": "%s?page_admin=preview" % request.path,

        "preview_html": preview_html,

        "markup_id_str": str(pagecontent.markup),

        "edit_page_form": edit_page_form,
        "pagecontent": pagecontent,
        "pagemeta": pagemeta,
    }
    return render_pylucid_response(request, 'page_admin/edit_inline_form.html', context,
        context_instance=RequestContext(request)
    )


def _edit_page_preview(request):
    """ AJAX preview """
    if request.method != 'POST':
        return HttpResponse("ERROR: Wrong request")
    edit_page_form = EditPageForm(request.POST)
    if not edit_page_form.is_valid():
        return HttpResponse("ERROR: Form not valid: %r" % edit_page_form.errors)
    content = edit_page_form.cleaned_data["content"]

    pagemeta, pagecontent = _get_pageobjects(request)

    raw_content = edit_page_form.cleaned_data["content"]
    html_content = apply_markup(raw_content, pagecontent.markup, request.page_msg, escape_django_tags=True)

    return HttpResponse(html_content)


@check_permissions(superuser_only=False,
    permissions=('pylucid.change_pagecontent', 'pylucid.change_pagemeta')
)
def http_get_view(request):
    action = request.GET["page_admin"]
    if action == "inline_edit":
        # replace the page content with the edit page form
        form_url = "%s?page_admin=inline_edit" % request.path
        return _edit_page(request, form_url)
    elif action == "preview":
        # preview via jQuery
        return _edit_page_preview(request)

    if settings.DEBUG:
        request.page_msg(_("Wrong get view parameter!"))



# coding: utf-8

"""
    PyLucid auth backends
    ~~~~~~~~~~~~~~~~~~~~~
    
    Limit user access to sites via UserProfile

    SiteAuthBackend:
        for normal username/plaintext password
        
    SiteSHALoginAuthBackend:
        for JS-SHA1-Login
    

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author$

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.utils.translation import ugettext as _
from django.contrib.auth.backends import ModelBackend

from pylucid_project.utils import crypt

from pylucid_project.apps.pylucid.shortcuts import failsafe_message
from pylucid_project.apps.pylucid.models import LogEntry


#LOCAL_DEBUG = True
LOCAL_DEBUG = False

if LOCAL_DEBUG:
    failsafe_message("Debug mode in auth_backends is on!", UserWarning)


def can_access_site(user):
    """
    Check if the user can access the current site.
    Use the UserProfile <-> site relationship.
    Skip check for all superusers.
    """
    if user.is_superuser:
        if LOCAL_DEBUG:
            failsafe_message("Superuser can access all sites.")
        return True
    else:
        if LOCAL_DEBUG:
            failsafe_message("No superuser -> check UserProfile.")

    try:
        user_profile = user.get_profile()
    except Exception, err:
        msg = _("Error getting user profile: %s") % err
        LogEntry.objects.log_action(app_label="pylucid", action="auth_backends", message=msg)
        failsafe_message(msg)
        return

    current_site = Site.objects.get_current()
    sites = user_profile.sites.all()
    if current_site in sites:
        if LOCAL_DEBUG:
            failsafe_message("User can access these site.")
        return True
    else:
        msg = _("You can't access these site!")
        LogEntry.objects.log_action(
            app_label="pylucid", action="auth_backends", message=msg,
            data={
                "user_username": user.username,
                "site:": current_site.name,
            }
        )
        failsafe_message(msg)
        return


class SiteAuthBackend(ModelBackend):
    """
    Normal username/plaintext password authentication, but we limit user to sites.
    """
    def authenticate(self, username=None, password=None):
        try:
            user = User.objects.get(username=username)
            if not user.check_password(password):
                if settings.DEBUG or LOCAL_DEBUG:
                    failsafe_message("Wrong password!")
                return
        except User.DoesNotExist, err:
            msg = _("User %(username)s doesn't exist: %(err)s") % {"username": username, "err":err}
            LogEntry.objects.log_action(
                app_label="pylucid", action="auth_backends", message=msg,
            )
            if LOCAL_DEBUG:
                raise
            if settings.DEBUG:
                failsafe_message()
            return

        if LOCAL_DEBUG:
            failsafe_message("Username %s and password ok." % username)

        # Limit the access to UserProfile <-> site relationship
        if can_access_site(user) == True:
            return user



class SiteSHALoginAuthBackend(ModelBackend):
    """
    Used for PyLucid JS-SHA-Login.
    Check challenge and limit access to sites.
    """
    def authenticate(self, user=None, challenge=None, sha_a2=None, sha_b=None, sha_checksum=None):
        if user == None: # Nothing to do: Normal auth?
            return

        try:
            check = crypt.check_js_sha_checksum(challenge, sha_a2, sha_b, sha_checksum)
        except crypt.SaltHashError, err:
            # Wrong password
            LogEntry.objects.log_action(
                app_label="pylucid", action="auth_backends",
                message="User %r check_js_sha_checksum error: %s" % (user, err),
            )
            if LOCAL_DEBUG:
                raise
            if settings.DEBUG:
                failsafe_message(err)
            return None

        if check != True:
            # Wrong password
            LogEntry.objects.log_action(
                app_label="pylucid", action="auth_backends",
                message="User %r check_js_sha_checksum failed." % user,
            )
            return

        # Limit the access to UserProfile <-> site relationship
        if can_access_site(user) == True:
            return user



# coding: utf-8

from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
from django.contrib import admin


class BaseAdmin(admin.ModelAdmin):
    def view_on_site_link(self, obj):
        """ view on site link in admin changelist, try to use complete uri with site info. """
        absolute_url = obj.get_absolute_url()
        if hasattr(obj, "get_absolute_uri"):
            url = obj.get_absolute_uri() # complete uri contains protocol and site domain.
        else:
            url = absolute_url

        context = {"absolute_url": absolute_url, "url": url}
        html = render_to_string('admin/pylucid/includes/view_on_site_link.html', context)
        return html

    view_on_site_link.short_description = _("View on site")
    view_on_site_link.allow_tags = True



# coding:utf-8

import re

CSS_RE = re.compile(r'#([a-f0-9]{6}) *;', re.IGNORECASE)
CSS_CONVERT_RE = re.compile(r'#([a-f0-9])([a-f0-9])([a-f0-9]) *;', re.IGNORECASE)


def filter_content(content):
    """
    Skip all lines, start with .pygments - Used in update routine.
    Note: result is not valid CSS!
    """
    result = []
    for line in content.splitlines():
        line = line.strip()
        if line and not line.startswith(".pygments"):
            result.append(line)
    return "\n".join(result)

def extract_colors(content, existing_color_dict=None):
    """
    # TODO: Preprocess: Change all 3 length values to 6 length values for merging it.
    
    >>> extract_colors(".foo { color: #000000; }")
    ('.foo { color: {{ color_0 }}; }', {'color_0': '000000'})
    
    it's case insensitivity:
    >>> extract_colors(".foo{color:#C9C573;} .bar{color:#c9c573;}")
    ('.foo{color:{{ color_0 }};} .bar{color:{{ color_0 }};}', {'color_0': 'c9c573'})
    
    Convert/merge 3 length values:
    >>> extract_colors(".foo{color:#11AAff;} .bar{color:#1af;}")
    ('.foo{color:{{ color_0 }};} .bar{color:{{ color_0 }};}', {'color_0': '11aaff'})
    
    You can give a existing color map:
    >>> extract_colors(".foo { color: #000000; }", existing_color_dict={"black":"000000"})
    ('.foo { color: {{ black }}; }', {'black': '000000'})
    
    Skip non color values:
    >>> extract_colors("to short #12345; to long #1234567; /* no # 123; color #aa11ff-value */")
    ('to short #12345; to long #1234567; /* no # 123; color #aa11ff-value */', {})
    """
    # Convert all 3 length values to 6 length
    new_content = CSS_CONVERT_RE.sub("#\g<1>\g<1>\g<2>\g<2>\g<3>\g<3>;", content)

    colors = set(CSS_RE.findall(new_content))

    if existing_color_dict:
        color_dict = existing_color_dict
        exist_color = dict([(v, k) for k, v in existing_color_dict.iteritems()])
    else:
        color_dict = {}
        exist_color = {}

    for no, color in enumerate(colors):
        color_lower = color.lower()

        if color_lower in exist_color:
            # color exist in other case
            key = exist_color[color_lower]
        else:
            # new color
            key = "color_%s" % no
            color_dict[key] = color_lower
            exist_color[color_lower] = key

        new_content = new_content.replace("#%s" % color, "{{ %s }}" % key)

    return new_content, color_dict


if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



# coding: utf-8

"""
    PyLucid JS-SHA-Login tests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~
    
    A secure JavaScript SHA-1 Login and a plaintext fallback login.
    
    Last commit info:
    ~~~~~~~~~
    $LastChangedDate:$
    $Rev:$
    $Author: JensDiemer $
    
    :copyleft: 2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

if __name__ == "__main__":
    # run unittest directly
    import os
#    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"
    os.environ["DJANGO_SETTINGS_MODULE"] = "pylucid_project.pylucid_plugins.auth.test_settings"
    virtualenv_file = "../../../../../../PyLucid_env/bin/activate_this.py"
    execfile(virtualenv_file, dict(__file__=virtualenv_file))

from django.test import TestCase

#class AuthTest(TestCase):
#    fixtures = ['pylucid.json']
#
#    def test_basic_addition(self):
#        """
#        Tests that 1 + 1 always equals 2.
#        """
#        self.failUnlessEqual(1 + 1, 2)
#
#__test__ = {"doctest": """
#Another way to test that 1 + 1 is equal to 2.
#
#>>> 1 + 1 == 2
#False
#"""}


from django.test.client import Client
from django_tools.unittest_utils import unittest_base, BrowserDebug

from pylucid_project.tests.test_tools import basetest


LOGIN_URL = "?auth=login"


class LoginTest(basetest.BaseUnittest):
    fixtures = ['pylucid.json']

    def setUp(self):
        self.client = Client() # start a new session

    def test_login_link(self):
        """ Simple check if login link exist. """
        response = self.client.get("/admin/", HTTP_ACCEPT_LANGUAGE="en")
        self.assertResponse(response,
            must_contain=(
                # PageContent:
                '1-rootpage content',
                '<title>1-rootpage title',
                # Login in link:
                '<span class="PyLucidPlugins auth" id="auth_lucidTag">',
                '<a href="?auth=login"',
                'id="login_link',
            ),
            must_not_contain=('Error', "traceback", 'Permission denied'),
        )

#    def test_login_get_form(self):
#        """ Simple check if login link exist. """
#        response = self.client.get(LOGIN_URL)
#        self.assertResponse(response,
#            must_contain=(
#                '<title>1-rootpage title', # PageContent
#                # Form stuff:
#                "JS-SHA-LogIn", "username", "sha_login", "next_url"
#            ),
#            must_not_contain=(
#                "Traceback", 'Permission denied'
#                '1-rootpage content', # PageContent
#            ),
#        )
#
#    def test_login_ajax_form(self):
#        """ Check if we get the login form via AJAX """
#        response = self.client.get(LOGIN_URL, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
#        self.failUnlessEqual(response.status_code, 200)
#        self.assertResponse(response,
#            must_contain=(
#                # Form stuff:
#                "JS-SHA-LogIn", "username", "sha_login", "next_url"
#            ),
#            must_not_contain=(
#                '<title>1-rootpage title', "<body", "<head>", # <- not a complete page
#                "Traceback", 'Permission denied'
#                '1-rootpage content', # PageContent
#            ),
#        )



if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
    management.call_command('test', __file__, verbosity=1)



# coding:utf-8

import os
import posixpath

import test_tools # before django imports!

#from django_tools.utils import info_print; info_print.redirect_stdout()

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType

from django_tools.unittest_utils import unittest_base, BrowserDebug

from pylucid_project.apps.pylucid.models import EditableHtmlHeadFile, PageMeta, PageContent
from pylucid_project.apps.pylucid.preference_forms import SystemPreferencesForm

from pylucid_project.tests.test_tools import basetest
from pylucid_project.tests import unittest_plugin


EDIT_PAGE_URL = "/?page_admin=inline_edit"
PREVIEW_URL = "/?page_admin=preview"
LOGIN_URL_PREFIX = "http://testserver/?auth=login"
LOGIN_URL = LOGIN_URL_PREFIX + "&next_url=/"
FORM_URL_PREFIX = "/pylucid_admin/plugins/page_admin"


class EditPageInlineTest(basetest.BaseUnittest):
    """ Test for editing a page inline. """
    def assertCanNotEdit(self, response):
        #BrowserDebug.debug_response(response)
        self.failUnlessEqual(response.status_code, 403)
        self.assertResponse(response,
            must_contain=('Permission denied',),
            must_not_contain=(
                '1-rootpage content', # PageContent
                '<title>1-rootpage title',
            ),
        )

    def assertCanEdit(self, response):
        """ assert that the response is the edit page form """
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                '1-rootpage content', # PageContent
                '<title>1-rootpage title',
                # Some form strings:
                'input type="submit" name="save" value="save"',
                'form action="/?page_admin=inline_edit"',
                'textarea id="id_content"',
            ),
            must_not_contain=(
                "Traceback", 'Permission denied',
            ),
        )

    def test_permissions_anonymous(self):
        """
        Test edit a page as a anonymous user
        The user should be redirected to a login page with next_url == EDIT_PAGE_URL
        """
        response = self.client.get(EDIT_PAGE_URL)
        self.assertRedirects(response, expected_url=LOGIN_URL, status_code=302)

    def test_permissions_superuser(self):
        """ Test edit a page as superuser """
        self.login(usertype="superuser")
        response = self.client.get(EDIT_PAGE_URL)
        self.assertCanEdit(response)

    def test_permissions_normal(self):
        """ A normal user must have explicit permissions to edit a page. """

        def check_permissions(permissions, assertion):
            user = self.login(usertype="normal")
            user.user_permissions.clear() # Delete all existing permissions
            for permission, model in permissions:
                # Give the test user the permissions:
                content_type = ContentType.objects.get_for_model(model)
                perm = Permission.objects.get(content_type=content_type, codename=permission)
                user.user_permissions.add(perm)

            user = self.login(usertype="normal") # "reloading" user to purge user._perm_cache 

            # low level check permissions:
            codenames = [u"pylucid.%s" % permission for permission, model in permissions]
            self.failUnless(user.has_perms(codenames),
                "Low level error, user has not permissions: %r, he has: %r" % (
                    codenames, user.get_all_permissions()
                )
            )
            # Check if he can now edit the page:
            response = self.client.get(EDIT_PAGE_URL)
            assertion(response)

        # normal user has no permissions
        check_permissions(permissions=[], assertion=self.assertCanNotEdit)

        # normal user has only one permission
        check_permissions(
            permissions=[("change_pagecontent", PageContent)],
            assertion=self.assertCanNotEdit
        )
        check_permissions(
            permissions=[("change_pagemeta", PageMeta)],
            assertion=self.assertCanNotEdit
        )

        # normal user has all permission -> he can edit the page
        check_permissions(
            permissions=[("change_pagecontent", PageContent), ("change_pagemeta", PageMeta)],
            assertion=self.assertCanEdit
        )

    def test_ajax_form(self):
        """ Test AJAX request of the edit page form """
        self.login(usertype="superuser")
        response = self.client.get(EDIT_PAGE_URL, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                '1-rootpage content', # PageContent
                # addition javascript from {% extrahead %} block:
                '<!-- extrahead from .../apps/pylucid/defaulttags/extraheadBlock.py',
                # Some form strings:
                'input type="submit" name="save" value="save"',
                'form action="/?page_admin=inline_edit"',
                'textarea id="id_content"',
            ),
            must_not_contain=(
                '<title>1-rootpage title', "<body", "<head>", # <- not a complete page
                "Traceback", 'Permission denied',
            ),
        )

    def test_ajax_preview(self):
        """ Test ajax edit page preview """
        self.login(usertype="superuser")
        response = self.client.post(PREVIEW_URL,
            {"content": "==headline"},
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        self.failUnlessEqual(response.status_code, 200)
        self.failUnlessEqual("<h2>headline</h2>\n", response.content)


class CreateNewContentTest(basetest.BaseUnittest):
    """ Test for creating new content/plugin page """

    def __init__(self, *args, **kwargs):
        super(CreateNewContentTest, self).__init__(*args, **kwargs)

        self.new_content_url = reverse("PageAdmin-new_content_page")
        self.new_plugin_url = reverse("PageAdmin-new_plugin_page")

    def test_new_plugin_permissions(self):
        """ anonymous user should be redirectet to the login page with a next_url. """
        response = self.client.get(self.new_plugin_url)
        self.failUnlessEqual(response.status_code, 302)
        self.failUnlessEqual(response['Location'],
            "http://testserver/?auth=login&next_url=/pylucid_admin/plugins/page_admin/new_plugin_page/"
        )

    def test_new_content_permissions(self):
        """ anonymous user should be redirectet to the login page with a next_url. """
        response = self.client.get(self.new_content_url)
        self.failUnlessEqual(response.status_code, 302)
        self.failUnlessEqual(response['Location'],
            "http://testserver/?auth=login&next_url=/pylucid_admin/plugins/page_admin/new_content_page/"
        )

    def test_new_content_form(self):
        """ Test if we get the "new content" input form """
        self.login(usertype="superuser")
        response = self.client.get(self.new_content_url)
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                "Create a new page",
                'form action="%s/new_content_page/" method="post" id="edit_page_form"' % FORM_URL_PREFIX,
                'input type="submit" name="save" value="save"',
                'textarea id="id_content"',
            ),
            must_not_contain=(
                "Traceback", 'Permission denied',
            ),
        )

# TODO:
#    def test_new_plugin_form(self):
#        """ Test if we get the "new plugin" input form """
#        self.login(usertype="superuser")
#        response = self.client.get(self.new_plugin_url)
#        self.failUnlessEqual(response.status_code, 200)
#        self.assertResponse(response,
#            must_contain=(
#                "Create a new plugin page",
#                'form action="%s/new_plugin_page/" method="post"' % FORM_URL_PREFIX,
#                'input type="submit" name="save" value="save"',
#                'select name="app_label" id="id_app_label"',
#                'option value="pylucid_project.pylucid_plugins.unittest_plugin"',
#            ),
#            must_not_contain=(
#                "Traceback", 'Permission denied',
#            ),
#        )

    def _test_create_page(self, form_url, page_type, extra_post_data, extra_must_contain):
        """
        Create a new page via POST.
        And check the created page via GET.
        """
        lang_code = self.default_lang_entry.code
        lang_desc = self.default_lang_entry.description
        for site in TestSites():
            info_string = "(lang: %s, site: %s)" % (lang_code, site.name)
            user = self.login(usertype="superuser")

            # Get the empty form
            response = self.client.get(form_url, HTTP_ACCEPT_LANGUAGE=lang_code)
            self.assertResponse(response,
                must_contain=(
                    "Create a new page",
                    "This new %s page will be create on site: '<strong>%s</strong>'" % (
                        page_type, site.domain
                    ),
                    "<legend>Language: <strong>%s</strong></legend>" % lang_desc,
                ),
                must_not_contain=('This field is required', "traceback")
            )

            post_data = {
                "position": "0",
                "design": "1",
                "slug": "new_%s_page_%s_%s" % (page_type, lang_code, site.name.replace(".", "_")),
                "name": "New %s page name %s" % (page_type, info_string),
                "title": "New %s page title %s" % (page_type, info_string),
                "robots": "follow and index me!",
                "keywords": "some, keywords, here ?",
                "description": "The %s page description. %s" % (page_type, info_string),
                "showlinks": "on",
            }
            for key, value in extra_post_data.items():
                post_data[key] = value % {"info_string": info_string}

            response = self.client.post(form_url, post_data, HTTP_ACCEPT_LANGUAGE=lang_code)
            self.assertResponse(response,
                must_not_contain=('This field is required', "error", "traceback")
            )
            #BrowserDebug.debug_response(response)

            new_page_url = "http://testserver/%s/%s/" % (lang_code, post_data["slug"])
            # Don't use self.assertRedirects here, because we can check the page_msg.
            self.failUnlessEqual(response.status_code, 302)
            self.failUnlessEqual(response['Location'], str(new_page_url)) # no unicode in headers

            # Check the new page
            response = self.client.get(new_page_url, HTTP_ACCEPT_LANGUAGE=lang_code)

            must_contain = [
                "New %s page" % page_type, "created.",
                post_data["name"],
                post_data["slug"],
                'meta name="robots" content="%s"' % post_data["robots"],
                'meta name="keywords" content="%s"' % post_data["keywords"],
                'meta name="description" content="%s"' % post_data["description"],
                '<title>%s' % post_data["title"],
                '<a href="/%s/%s/">%s</a>' % (lang_code, post_data["slug"], post_data["title"])
            ]
            for extra in extra_must_contain:
                must_contain.append(extra % {"info_string": info_string})

            self.assertResponse(response, must_contain,
                must_not_contain=(
                    "Traceback", 'Permission denied', 'This field is required',
                ),
            )

    def test_create_new_content_page(self):
        """ Create a new content page via POST and check the created page. """
        extra_post_data = {
            "content": "New page content %(info_string)s",
            "markup": "0",
        }
        extra_must_contain = [
            extra_post_data["content"]
        ]
        self._test_create_page(self.new_content_url, "content", extra_post_data, extra_must_contain)


#    def test_create_new_plugin_page(self):
#        """
#        Create a new plugin page via POST and check the created page.
#        Use the unittest_plugin.views.view_root() for this.
#        """
#        extra_post_data = {
#            "app_label": "pylucid_project.pylucid_plugins.unittest_plugin",
#            "urls_filename": "urls.py",
#        }
#        extra_must_contain = [
#            unittest_plugin.views.PLUGINPAGE_ROOT_STRING_RESPONSE
#        ]
#        self._test_create_page(self.new_plugin_url, "plugin", extra_post_data, extra_must_contain)


if __name__ == "__main__":
    # Run this unitest directly
#    from django_tools.utils import info_print; info_print.redirect_stdout()

#    unittest_base.direct_run(__file__) # run all test from this file

    from django.core import management
    management.call_command('test', "test_PluginEditPage.CreateNewContentTest")



 # coding: utf-8

"""
    PyLucid page messages
    ~~~~~~~~~~~~~~~~~~~~~

    A small Wrapper aound djangos user messages system.

    enhanced features:
      - easy callable to print a messages
      - simple color output: self.page_msg.successful()
      - use pprint for dicts and lists
      - special debug mode:
          Inserts informationen, where from the message has come.

    Can also used in Plugin for storing internal messages.

    Links
    ~~~~~
    http://www.pylucid.org/_goto/134/plugin-output/
    http://www.djangoproject.com/documentation/authentication/#messages

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: 2009-11-19 16:53:11 +0100 (Do, 19. Nov 2009) $
    $Rev: 2395 $
    $Author: JensDiemer $

    :copyleft: 2008-2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os
import sys
import pprint
import inspect
import warnings
import traceback

from django.conf import settings
from django.utils.encoding import smart_str
from django.utils.safestring import SafeData, mark_safe
from django.template.loader import render_to_string

from pylucid_project.utils.escape import escape
from pylucid_project.middlewares.utils import replace_content, cut_filename

TAG = u"<!-- page_messages -->"

SESSION_KEY = "PAGE_MESSAGES"


STACK_LIMIT = 6


class PageMessages(object):
    """
    The page message container.
    """
    def __init__(self, request):
        self.request = request

        self.debug_mode = settings.DEBUG
        self._charset = settings.DEFAULT_CHARSET

        try:
            session = request.session
        except AttributeError, err:
            raise AttributeError("Can't the sessin object."
                " PageMessagesMiddleware must be added *after* SessionMiddleware!"
                " original error was: %s" % err)

        if SESSION_KEY not in session:
            # Add a page_msg list into session
            session[SESSION_KEY] = []
        elif session[SESSION_KEY] != []:
            # There exist old messages from previous requests -> add a seperator after them. 
            self.info(mark_safe("<hr />"))

    def get_and_delete_messages(self):
        """ delete messages from session and return them """
        django_msg = self.request.user.get_and_delete_messages()
        for msg in django_msg:
            # Append messages from django
            self.info(msg)

        msg = self.request.session[SESSION_KEY]
        # remove "old" page_msg from session
        del(self.request.session[SESSION_KEY])
        return msg

    #_________________________________________________________________________

    def write(self, *msg):
        self.append_message("info", *msg)

    def __call__(self, *msg):
        self.append_message("info", *msg)

    def successful(self, *msg):
        self.append_message("successful", *msg)

    def info(self, *msg):
        self.append_message("info", *msg)

    def warning(self, *msg):
        self.append_message("warning", *msg)

    def error(self, *msg):
        self.append_message("error", *msg)

    def critical(self, *msg):
        self.append_message("critical", *msg)

    def insert_traceback(self):
        """ Insert last traceback, if DEBUG==True """
        if not settings.DEBUG:
            return

        exc = traceback.format_exc()
        msg = mark_safe("<pre>%s</pre>" % exc)
        self.error(msg)

    #_________________________________________________________________________

    def showwarning(self, message, category, filename, lineno, file=None, line=None):
        """ for redirecting warnings """
        old_debug_mode = self.debug_mode
        self.debug_mode = False # no stack_info would be added
        self.warning(str(message))
        self.debug_mode = old_debug_mode

        # Add stack_info
        block_data = self.request.session[SESSION_KEY][-1]
        filename = u"..." + filename[-30:]
        block_data["stack_info"] = "(%s from %s - line %s)" % (category.__name__, filename, lineno)

        if file is not None:
            try:
                file.write(warnings.formatwarning(message, category, filename, lineno, line))
            except IOError:
                pass

    #_________________________________________________________________________

    def append_message(self, msg_type, *msg):
        """
        Add a message with type info.
        -if debug_mode is on: insert a info from where the message sended.
        -for dict, list use pprint ;)
        """
        lines = []
        for raw_item in msg:
            if isinstance(raw_item, dict) or isinstance(raw_item, list):
                item = pprint.pformat(raw_item)
                raw_lines = item.split("\n")
                for raw_line in raw_lines:
                    lines.append(self.encode_and_prepare(raw_line))
                lines.append("")
            else:
                item = self.encode_and_prepare(raw_item)
                pos = len(lines)
                if pos == 0:
                    lines.append(item)
                else:
                    lines[pos - 1] += " " + item

        block_data = {
            "msg_type": msg_type,
            "lines": lines,
        }

        if self.debug_mode == True:
            block_data["full_url"] = self.request.get_full_path()
            block_data["stack_info"] = self._get_stack_info()

        if SESSION_KEY not in self.request.session:
            # exemption: call page_msg directly after logout (session deleted)
            self.request.session[SESSION_KEY] = []

        self.request.session[SESSION_KEY].append(block_data)
        self.request.session.modified = True # FIXME: Don't really know why this is needed

    def _get_stack_info(self):
        """
        return stack_info: Where from the announcement comes?
        """
        self_basename = os.path.basename(__file__)
        if self_basename.endswith(".pyc"):
            # cut: ".pyc" -> ".py"
            self_basename = self_basename[:-1]

        stack_list = inspect.stack()
        # go forward in the stack, to outside of this file.
        for no, stack_line in enumerate(stack_list):
            filename = stack_line[1]
            if os.path.basename(filename) != self_basename:
                break

        stack_list = stack_list[no:no + STACK_LIMIT] # limit the displayed stack info

        stack_info = []
        for stack_line in reversed(stack_list):
            filename = cut_filename(stack_line[1])
            lineno = stack_line[2]
            func_name = stack_line[3]
            stack_info.append("%s %4s %s" % (filename, lineno, func_name))

        return "\\n\\n".join(stack_info)

    def encode_and_prepare(self, txt):
        """ prepare the given text """
        if isinstance(txt, SafeData):
            # pass string witch marked with django.utils.safestring.mark_safe
            return txt
        elif isinstance(txt, unicode):
            # encode unicode strings
            return smart_str(txt, encoding=self._charset)
        elif isinstance(txt, str):
            # pass normal strings
            return txt
        else:
            # return the printable representation of an object
            try:
                # e.g.: django ugettext_lazy string?
                return smart_str(unicode(txt), encoding=self._charset)
            except:
                return repr(txt)


    #________________________________________________________________

    def add_test_lines(self):
        """ Add many test lines """
        self.append_message("successful", "test successful line")
        self.append_message("info", "test info line")
        self.append_message("warning", "test warning line")
        self.append_message("error", "test error line")
        self.append_message("critical", "test critical line")
        self.append_message("info", "Here comes:", ["a", "messages", "list"], "and text after the list.")
        self.append_message("info", "Here is:", {"a":"dict"}, "and text after a dict.")
        self.append_message("warning", "<html characters should be escaped.>")
        self.append_message("warning", mark_safe("You can use <strong>mark_safe</strong> to use html code."))

    def __repr__(self):
        return "page messages: %s" % repr(self.request.session[SESSION_KEY])

    def __str__(self):
        return "pages messages: %s" % ", ".join(self.request.session[SESSION_KEY])

    def __unicode__(self):
        return u"page messages: %s" % u", ".join(self.request.session[SESSION_KEY])

    def __len__(self):
        return len(self.request.session[SESSION_KEY])




class PageMessagesMiddleware(object):
    def __init__(self):
        self.old_showwarning = warnings.showwarning
        self.tag = smart_str(TAG, encoding=settings.DEFAULT_CHARSET)

    def process_request(self, request):
        """ add page_msg object to request object """
        request.page_msg = PageMessages(request)

        # redirect warnings into page_msg
        warnings.showwarning = request.page_msg.showwarning

        #request.page_msg.add_test_lines()

    def process_response(self, request, response):
        """
        insert all page messages into the html page.
        """
        warnings.showwarning = self.old_showwarning

        if not hasattr(request, "session") or SESSION_KEY not in request.session:
            # There exist no session or page_msg -> do nothing
            return response

        if not "html" in response._headers["content-type"][1]:
            # No HTML Page -> do nothing
            return response

        content = response.content
        if self.tag not in content:
            # We can't replace the TAG with the page_msg if it's not in the content ;)
            # Do nothing and try it in the next request
            return response

        page_msg = request.page_msg
        if len(page_msg) == 0:
            # There exists no page messages
            message_string = ""
        else:
            request._cache_update_cache = False # Don't cache pages with page_msg

            # Get the messages and delete it from the session
            msg_list = page_msg.get_and_delete_messages()
            context = {
                "msg_list": msg_list,
                "stack_limit": STACK_LIMIT,
            }
            message_string = render_to_string("pylucid/page_msg.html", context)
            message_string = smart_str(message_string, encoding=settings.DEFAULT_CHARSET)

        response = replace_content(response, self.tag, message_string)

        return response



# coding: utf-8

"""
    PyLucid OpenStreetMap plugin
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~
    $LastChangedDate:$
    $Rev:$
    $Author: JensDiemer $

    :copyleft: 2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

from django.db import models
from django.core import urlresolvers
from django.db.models import signals
from django.utils.safestring import mark_safe
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _

# http://code.google.com/p/django-tools/
from django_tools.middlewares import ThreadLocal

from pylucid_project.apps.pylucid.models import PageContent
from pylucid_project.apps.pylucid.markup.converter import apply_markup
from pylucid_project.apps.pylucid.models.base_models import AutoSiteM2M, UpdateInfoBaseModel

class MapEntry(AutoSiteM2M, UpdateInfoBaseModel):
    name = models.SlugField(help_text=_("Short name for lucidTag"))
    
    width = models.PositiveIntegerField(
        default=450,
        help_text=_("Map width size")
    )
    height = models.PositiveIntegerField(
        default=350,
        help_text=_("Map height size")
    )
    
    # http://www.openstreetmap.org/?lat=51.430123&lon=6.774099&zoom=18&layers=B000FTF
    lon = models.FloatField(
        help_text=_("map geographic longitude (vertically) coordinate"),
    )
    lat = models.FloatField(
        help_text=_("map geographic latitude (horizontally) coordinate"),
    )
    zoom = models.PositiveSmallIntegerField(
        default=12,
        help_text=_("Map zoom factor"),
    )
    
    marker_lon = models.FloatField(
        help_text=_("Text marker longitude (vertically) coordinate")
    )
    marker_lat = models.FloatField(
        help_text=_("Text marker latitude (horizontally) coordinate")
    )
    marker_width = models.PositiveIntegerField(
        default=180,
        help_text=_("Text marker width size")
    )
    marker_height = models.PositiveIntegerField(
        default=125,
        help_text=_("Text marker height size")
    )
    marker_text=models.TextField(help_text=_("Marker text"))
    markup = models.IntegerField(max_length=1,
        default=PageContent.MARKUP_CREOLE,
        choices=PageContent.MARKUP_CHOICES
    )
    
    def get_html(self):
        """ return the marker_text as html """
        request = ThreadLocal.get_current_request()
        html = apply_markup(
            raw_content=self.marker_text,
            markup_no=self.markup, 
            page_msg=request.page_msg
        )
        # Needed for JavaScript:
        html = html.strip().replace("\n", "").replace('"', '\"')
        return mark_safe(html)


# coding: utf-8

"""
    PyLucid.cache
    ~~~~~~~~~~~~~

    Cleanup cache after PageTree, PageMeta or PageContent changed.
    
    Cleaning the cache is not only necessary so that the client
    sees the updates also. If a page is no longer accessable for
    anonymous, they should no longer be delivered from the cache.
    
    We clean the complete page cache, because we can't known witch
    part of the cache has been really changed. e.g. the update journal
    is somewhere or if the PageTree hierarchy change...
       
    We used the middlewares:
        django.middleware.cache.UpdateCacheMiddleware
        django.middleware.cache.FetchFromCacheMiddleware
    see also: http://docs.djangoproject.com/en/dev/topics/cache/#the-per-site-cache
    
    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author$

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.utils.cache import _generate_cache_header_key

from django.conf import settings
from django.core.cache import cache
from django.contrib.sites.models import Site

from django_tools.middlewares import ThreadLocal

from pylucid_project.apps.pylucid.shortcuts import failsafe_message



class FakeRequest(object):
    """ Used to get the cache key from django.utils.cache._generate_cache_header_key """
    def __init__(self, path):
        self.path = path



def clean_complete_pagecache(sender, **kwargs):
    """
    signal post save handler -> clean all page cache entries
    We simply delete all entries from the cache on this site.
    
    We only cleanup the cache one time in one request.
    We store the information "Cache has been deleted" in the request._cache_cleaned bool.
    In e.g. PyLucid "edit all" view, PageTree, PageMeta and PageContent are stored in series.
    """
    from pylucid_project.apps.pylucid.models import PageMeta # import here, against import loops

    model_instance = kwargs["instance"]

    request = ThreadLocal.get_current_request()
    if request is None:
        if settings.DEBUG:
            failsafe_message("Info: Skip page cache cleanup.")
        return

    verbose = settings.DEBUG or request.user.is_superuser

    cache_cleaned = getattr(request, "_cache_cleaned", False)
    if cache_cleaned:
        # Cache has been deleted in this request.
        if verbose:
            request.page_msg.info(
                "%r was saved. But cache was cleaned in the past, ok." % model_instance
            )
        return

    current_site = Site.objects.get_current()

    queryset = PageMeta.objects.select_related() # PageTree is needed, so select related objects.

    # Cleanup only the current site
    queryset = queryset.filter(pagetree__site=current_site)

    # Use the same key prefix as in django.middleware.cache.FetchFromCacheMiddleware
    key_prefix = settings.CACHE_MIDDLEWARE_KEY_PREFIX

    total_count = 0
    cleaned = 0
    for pagemeta in queryset:
        absolute_url = pagemeta.get_absolute_url()

        # Use the same routine as in django.middleware.cache.FetchFromCacheMiddleware
        fake_request = FakeRequest(path=absolute_url)
        cache_key = _generate_cache_header_key(key_prefix, fake_request)

        if verbose: # Count some information only in verbose mode
            total_count += 1
            is_in_cache = cache.get(cache_key, None) is not None
            if is_in_cache:
                cleaned += 1

        cache.delete(cache_key)

    # store the information "Cache has been deleted" for later signals
    #if request is not None:
    request._cache_cleaned = True

    if verbose:
        msg = (
            "%r was saved, clean page cache:"
            " %s cachable items on this site."
            " %s items deleted from cache."
        ) % (model_instance, total_count, cleaned)
        request.page_msg.successful(msg)



# coding: utf-8
"""
    PyLucid
    ~~~~~~~

    A Python based Content Management System
    written with the help of the powerful
    Webframework Django.

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author$

    :copyleft: 2009-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

__version__ = (0, 9, 0, 'RC2')

try:
    from django.utils.version import get_svn_revision
except ImportError:
    pass
else:
    path = os.path.split(os.path.abspath(__file__))[0]
    svn_revision = get_svn_revision(path)
    if svn_revision != u'SVN-unknown':
        svn_revision = svn_revision.replace("-", "").lower()
        __version__ += (svn_revision,)


VERSION_STRING = '.'.join(str(part) for part in __version__)


if __name__ == "__main__":
    print VERSION_STRING



# coding:utf-8

import os
import time
from datetime import datetime, timedelta


if __name__ == "__main__":
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"
    virtualenv_file = "../../../../../bin/activate_this.py"
    execfile(virtualenv_file, dict(__file__=virtualenv_file))


from django import http
from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.sites.models import Site
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from django.contrib.sessions.models import Session

from dbtemplates.models import Template

from pylucid_project.apps.pylucid.models import LogEntry
from pylucid_project.apps.pylucid.decorators import check_permissions, render_to
from pylucid_project.apps.pylucid.markup.hightlighter import make_html, get_pygments_css

from pylucid_project.apps.pylucid_admin.admin_menu import AdminMenu

from tools.forms import HighlightCodeForm, CleanupLogForm, SelectTemplateForm


MYSQL_ENCODING_VARS = (
    "character_set_server", "character_set_connection", "character_set_results", "collation_connection",
)


def install(request):
    """ insert PyLucid admin views into PageTree """
    output = []

    admin_menu = AdminMenu(request, output)
    menu_section_entry = admin_menu.get_or_create_section("tools")

    admin_menu.add_menu_entry(
        parent=menu_section_entry,
        name="highlight code", title="highlight sourcecode with pygments",
        url_name="Tools-highlight_code"
    )
    admin_menu.add_menu_entry(
        parent=menu_section_entry,
        name="cleanup log table", title="Cleanup the log table",
        url_name="Tools-cleanup_log"
    )
    admin_menu.add_menu_entry(
        parent=menu_section_entry,
        name="cleanup session table", title="Cleanup the session table",
        url_name="Tools-cleanup_session"
    )
    admin_menu.add_menu_entry(
        parent=menu_section_entry, name="overwrite template",
        title="Overwrite a filesystem template with a new database headfile entry",
        url_name="Tools-overwrite_template"
    )
    return "\n".join(output)

#-----------------------------------------------------------------------------

@check_permissions(superuser_only=False, permissions=(u'pylucid.change_pagecontent',))
@render_to("tools/highlight_code.html")
def highlight_code(request):
    """ hightlight sourcecode for copy&paste """
    context = {
        "title": _("hightlight sourcecode"),
        "form_url": request.path,
    }

    # get the EditableHtmlHeadFile path to pygments.css (page_msg created, if not exists)
    pygments_css_path = get_pygments_css(request)
    context["pygments_css"] = pygments_css_path

    if request.method == "POST":
        form = HighlightCodeForm(request.POST)
        if form.is_valid():
            sourcecode = form.cleaned_data["sourcecode"]
            source_type = form.cleaned_data["source_type"]

            highlighted = make_html(sourcecode, source_type, django_escape=True)
            context["highlighted"] = highlighted

            html_code = make_html(highlighted, "html", django_escape=True)
            context["html_code"] = html_code
    else:
        form = HighlightCodeForm()

    context["form"] = form
    return context



@check_permissions(superuser_only=True)
@render_to("tools/cleanup_log.html")
def cleanup_log(request):
    """ Delete old log entries """
    context = {
        "title": _("Delete old log entries"),
        "form_url": request.path,
        "count_on_site": LogEntry.on_site.count(),
        "count_total": LogEntry.objects.count(),
        "oldest_on_site_entry": LogEntry.on_site.all().only("createtime").order_by('createtime')[0],
        "oldest_total_entry": LogEntry.objects.all().only("createtime").order_by('createtime')[0],
    }
    if request.method == "POST":
        form = CleanupLogForm(request.POST)
        if form.is_valid():
            start_time = time.time()
            number = form.cleaned_data["number"]
            delete_type = form.cleaned_data["delete_type"]
            limit_site = form.cleaned_data["limit_site"]

            if limit_site:
                queryset = LogEntry.on_site
            else:
                queryset = LogEntry.objects

            queryset = queryset.order_by('-createtime')

            if delete_type == CleanupLogForm.LAST_NUMBERS:
                ids = tuple(queryset[number:].values_list('id', flat=True))
                queryset = queryset.filter(id__in=ids)
            else:
                if delete_type == CleanupLogForm.LAST_DAYS:
                    delta = timedelta(days=number)
                elif delete_type == CleanupLogForm.LAST_HOURS:
                    delta = timedelta(hours=number)
                else:
                    raise AssertionError("Wrong delete_type") # should never happen

                now = datetime.now()
                datetime_filter = now - delta
                queryset = queryset.exclude(createtime__gte=datetime_filter)

            delete_count = queryset.count()
            queryset.delete()
            duration_time = time.time() - start_time
            request.page_msg(_("Delete %(count)s entries in %.2(duration)fsec") % {
                "count": delete_count, "duration":duration_time
            })
            return HttpResponseRedirect(request.path)
    else:
        form = CleanupLogForm()

    context["form"] = form
    return context


@check_permissions(superuser_only=True)
@render_to("tools/cleanup_session.html")
def cleanup_session(request):
    """ Delete old session entries """

    count_before = Session.objects.count()
    start_time = time.time()
    Session.objects.filter(expire_date__lt=datetime.now()).delete()
    duration_time = time.time() - start_time
    count_after = Session.objects.count()

    delete_count = count_before - count_after
    request.page_msg(_("Delete %(count)s entries in %.2(duration)fsec") % {
        "count": delete_count, "duration":duration_time
    })

    context = {
        "title": _("Delete old session entries"),
        "count_before": count_before,
        "count_after": count_after,
        "count_deleted": delete_count,
    }
    return context


#-----------------------------------------------------------------------------------------------------------
# overwrite template


class TemplateFile(object):
    def __init__(self, request, fs_path):
        self.request = request
        self.fs_path = fs_path

        self.name = fs_path.rsplit("templates", 1)[1].lstrip("/")

    def _get_fs_content(self):
        try:
            f = file(self.fs_path, "r")
            content = f.read()
            f.close()
        except Exception, err:
            request.page_msg.error("Can't read file: %s" % err)
        else:
            return content

    def get_or_create_dbtemplate(self):
        """
        create a dbtemplate entry with the content form filesystem.
        return the dbtemplate instance if success, otherwise: create a page_msg and return None
        """
        content = self._get_fs_content()
        if not content:
            # Content can't readed.
            return

        template, created = Template.objects.get_or_create(name=self.name,
            defaults={"content": content}
        )
        if created:
            template.save()
            current_site = Site.objects.get_current()
            template.sites.add(current_site)
            template.save()
        return template, created

    def get_content_preview(self):
        content = self._get_fs_content()
        if not content:
            # Can't read the template content, page_msg was created.
            return

        ext = os.path.splitext(self.fs_path)[1]
        html = make_html(content, ext, django_escape=True)
        return html



@check_permissions(
    superuser_only=False,
    permissions=(u'dbtemplates.add_template', u'dbtemplates.change_template')
)
@render_to("tools/overwrite_template.html")
def overwrite_template(request):
    """
    Overwrite a template:
    1. The user can choose between all existing template in filesystem.
    2. Read the content from filesystem and create a new dbtemplate entry.
    3. redirect to edit the nre dbtemplate entry
    """
    context = {
        "title": _("overwrite template"),
    }

    if request.method != "POST":
        form = SelectTemplateForm()
    else:
        form = SelectTemplateForm(request.POST)
        if form.is_valid():
            fs_path = form.cleaned_data["template"]
            template = TemplateFile(request, fs_path)

            if "preview" in request.POST:
                # Display only the template content
                preview_html = template.get_content_preview()
                if preview_html:
                    context["template"] = template

                    # get the EditableHtmlHeadFile path to pygments.css (page_msg created, if not exists)
                    pygments_css_path = get_pygments_css(request)
                    context["pygments_css"] = pygments_css_path
            else:
                # A new dbtemplate should be created
                instance, created = template.get_or_create_dbtemplate()
                if instance:
                    if created:
                        # New dbtemplate instance created -> edit it
                        # if instance == None: e.g.: error reading file -> page_msg was created
                        msg = _("New dbtemplate entry %s created.") % instance
                        LogEntry.objects.log_action(
                            app_label="pylucid_plugin.extrahead",
                            action="overwrite template %s" % template.name,
                            request=request,
                            message=msg
                        )
                    else:
                        msg = _("dbtemplate entry %s already exists!") % instance

                    msg += _(" You can edit it now.")

                    request.page_msg(msg)

                    # redirect to edit the new dbtemplate entry
                    url = reverse("admin:dbtemplates_template_change", args=(instance.id,))
                    return http.HttpResponseRedirect(url)

    context["form"] = form
    return context


if __name__ == "__main__":
    templates = [TemplateDir(dir) for dir in settings.TEMPLATE_DIRS]
